@@ -19,6 +19,7 @@ If not, see https://www.gnu.org/licenses/ .
 
 #include "functions.h"
 #include "preferences.h"
+#include "instrumentdata.h"
 #include "tools/fitgauss1d.h"
 
 #include <wcs.h>
@@ -58,8 +59,10 @@ QStringList datadir2StringList(QLineEdit *lineEdit)
 
 QString get_fileparameter(QFile *file, QString parametername, QString warn)
 {
+    if (file->fileName().isEmpty()) return "";  // Suppressing an error when the user launches THELI for the very first time
+
     if(!file->open(QIODevice::ReadOnly)) {
-        qDebug() << "get_fileparameter: "+file->fileName()+" "+file->errorString();
+        qDebug() << __func__ << file->fileName() << file->errorString();
         return "";
     }
 
@@ -486,7 +489,7 @@ QString findExecutableName(QString program)
     QStringList swarplist = {"swarp", "Swarp", "SWarp"};
     QStringList anetlist1 = {"solve-field"};
     QStringList anetlist2 = {"build-astrometry-index"};
-    QStringList pythonlist = {"python3", "python", "python2"};
+    QStringList pythonlist = {"python3", "python"};
 
     QString commandname = "";
     if (program == "source-extractor") {
@@ -510,12 +513,22 @@ QString findExecutableName(QString program)
     else if (program == "solve-field") {
         for (auto &it : anetlist1) {
             commandname = QStandardPaths::findExecutable(it);
+            if (commandname.isEmpty()) {
+                QStringList paths;
+                paths << "/usr/local/Astrometry/bin/";
+                commandname = QStandardPaths::findExecutable(it, paths);
+            }
             if (!commandname.isEmpty()) break;
         }
     }
     else if (program == "build-astrometry-index") {
         for (auto &it : anetlist2) {
             commandname = QStandardPaths::findExecutable(it);
+            if (commandname.isEmpty()) {
+                QStringList paths;
+                paths << "/usr/local/Astrometry/bin/";
+                commandname = QStandardPaths::findExecutable(it, paths);
+            }
             if (!commandname.isEmpty()) break;
         }
     }
@@ -1401,6 +1414,9 @@ bool readData3D(QString path, QVector<double> &x, QVector<double> &y, QVector<do
 // Angles are always passed / returned in degrees
 double getPosAnglefromCD(double cd11, double cd12, double cd21, double cd22, bool lock)
 {
+    // In case CD matrix is undefined (e.g. binned images)
+    if (cd11 == 0. && cd12 == 0. && cd21 == 0. && cd22 == 0.) return 0.;
+
     // the pixel scale
     double pscale1 = sqrt(cd11 * cd11 + cd21 * cd21);
     double pscale2 = sqrt(cd12 * cd12 + cd22 * cd22);
@@ -1465,7 +1481,7 @@ double getPosAnglefromCD(double cd11, double cd12, double cd21, double cd22, boo
             }
 
             else {
-                qDebug() << "getPosAnglefromCD(): ERROR: Could not determine position angle from CD matrix!";
+                qDebug() << __func__ << "ERROR: Could not determine position angle from CD matrix!";
             }
         }
     }
@@ -1493,14 +1509,15 @@ void rotateCDmatrix(double &cd11, double &cd12, double &cd21, double &cd22, doub
     double f22 = 1.;
 
     // unflip the matrix
-    matrix_mult_T(f11, f12, f21, f22, cd11, cd12, cd21, cd22);
+    matrixMult_T(f11, f12, f21, f22, cd11, cd12, cd21, cd22);
 
     // rotate the matrix to the new position angle
     double dPA = rad * (PAnew - PAold);
-    matrix_mult_T(cos(dPA), -sin(dPA), sin(dPA), cos(dPA), cd11, cd12, cd21, cd22);
+
+    matrixMult_T(cos(dPA), -sin(dPA), sin(dPA), cos(dPA), cd11, cd12, cd21, cd22);
 
     // flip the matrix
-    matrix_mult_T(f11, f12, f21, f22, cd11, cd12, cd21, cd22);
+    matrixMult_T(f11, f12, f21, f22, cd11, cd12, cd21, cd22);
 }
 
 // Angles are always passed / returned in degrees
@@ -1559,7 +1576,7 @@ int gauss_f(const gsl_vector* x, void* params, gsl_vector* f)
     return GSL_SUCCESS;
 }
 
-int gauss_df (const gsl_vector *x, void *params, gsl_matrix *J)
+int gauss_df(const gsl_vector *x, void *params, gsl_matrix *J)
 {
     size_t nx = ((struct dataGSL *)params)->nx;
     size_t ny = ((struct dataGSL *)params)->ny;
@@ -1583,10 +1600,87 @@ int gauss_df (const gsl_vector *x, void *params, gsl_matrix *J)
     return GSL_SUCCESS;
 }
 
-int gauss_fdf (const gsl_vector *x, void *params, gsl_vector *f, gsl_matrix *J)
+int gauss_fdf(const gsl_vector *x, void *params, gsl_vector *f, gsl_matrix *J)
 {
     gauss_f(x, params, f);
     gauss_df(x, params, J);
 
     return GSL_SUCCESS;
 }
+
+void flipData(QVector<float> &data, const QString dir, const int naxis1, const int naxis2)
+{
+    QVector<float> copy;
+    copy.resize(naxis1*naxis2);
+
+    long k = 0;
+    if (dir == "x") {
+        for (int j=0; j<naxis2; ++j) {
+            for (int i=0; i<naxis1; ++i) {
+                copy[k++] = data[naxis1-1-i + naxis1*j];
+            }
+        }
+    }
+    else if (dir == "y") {
+        for (int j=0; j<naxis2; ++j) {
+            for (int i=0; i<naxis1; ++i) {
+                copy[k++] = data[i + naxis1*(naxis2-1-j)];
+            }
+        }
+    }
+    else if (dir == "xy") {
+        for (int j=0; j<naxis2; ++j) {
+            for (int i=0; i<naxis1; ++i) {
+                copy[k++] = data[naxis1-i-1+naxis1*(naxis2-j-1)];
+            }
+        }
+    }
+
+    else {
+        // nothing yet
+    }
+
+    data.swap(copy);
+}
+
+void flipSections(QVector<long> &sections, const QString dir, const int naxis1, const int naxis2)
+{
+    QVector<long> copy = sections;
+
+    if (dir == "x") {
+        copy[1] = naxis1-1-sections[0];
+        copy[0] = naxis1-1-sections[1];
+    }
+    else if (dir == "y") {
+        copy[3] = naxis2-1-sections[2];
+        copy[2] = naxis2-1-sections[3];
+    }
+    else if (dir == "xy") {
+        copy[1] = naxis1-1-sections[0];
+        copy[0] = naxis1-1-sections[1];
+        copy[3] = naxis2-1-sections[2];
+        copy[2] = naxis2-1-sections[3];
+    }
+
+    sections = copy;
+}
+
+// In case the user has deactivated some chips, we need to find one that is valid
+int getValidChip(const instrumentDataType *instData)
+{
+    /*
+    int testChip = -1;
+    for (int chip=0; chip<instData->numChips; ++chip) {
+        if (!instData->badChips.contains(chip)) {
+            testChip = chip;
+            break;
+        }
+    }
+    if (testChip == -1) {
+        qDebug() << __func__ << "error: no data left after filtering";
+    }
+    return testChip;
+    */
+    if (instData->goodChips.isEmpty()) return 0;       // error message is printed by MainWindow::updateExcludedDetectors(QString badDetectors)
+    else return instData->goodChips[0];                // return the first valid chip
+}
