@@ -68,6 +68,7 @@ void IView::initDynrangeDrag()
             scene->removeItem(it);
         }
     }
+    if (scene->crosshairShown) scene->removeCrosshair();
 }
 
 void IView::showCurrentMousePos(QPointF point)
@@ -83,14 +84,15 @@ void IView::showCurrentMousePos(QPointF point)
 
     float x_cursor;
     float y_cursor;
+    // constant offsets required to show same coordinates as in skycat / ds9
     x_cursor = point.x() + 0.5;
     y_cursor = naxis2 - point.y() + 0.5;
 
     // Display the information
     QString xpos = QString::number(x_cursor);
     QString ypos = QString::number(y_cursor);
-    icdw->ui->xposLabel->setText("x = "+xpos);
-    icdw->ui->yposLabel->setText("y = "+ypos);
+    icdw->ui->xposLabel->setText(xpos);
+    icdw->ui->yposLabel->setText(ypos);
 
     // Pixel index in the 2D image
     long i = x_cursor - 0.5;
@@ -98,20 +100,27 @@ void IView::showCurrentMousePos(QPointF point)
     if (i<naxis1 && i>=0 && j<naxis2 && j>=0) {
         if (displayMode == "FITSmonochrome" || displayMode == "MEMview") {
             QString value = QString::number(fitsData[i+naxis1*j]);
-            icdw->ui->valueLabel->setText("Value = "+value);
+            icdw->ui->valueLabel->setText(value);
         }
         else {
             // Color FITS
             QString rval = QString::number(fitsDataR[i+naxis1*j]);
             QString gval = QString::number(fitsDataG[i+naxis1*j]);
             QString bval = QString::number(fitsDataB[i+naxis1*j]);
-            icdw->ui->valueLabel->setText("Value R = "+rval);
-            icdw->ui->valueGreenLabel->setText("Value G = "+gval);
-            icdw->ui->valueBlueLabel->setText("Value B = "+bval);
+            icdw->ui->valueLabel->setText(rval);
+            icdw->ui->valueGreenLabel->setText(gval);
+            icdw->ui->valueBlueLabel->setText(bval);
         }
     }
     else {
-        icdw->ui->valueLabel->setText("Value = ");
+        if (displayMode == "FITSmonochrome" || displayMode == "MEMview") {
+            icdw->ui->valueLabel->setText("");
+        }
+        else {
+            icdw->ui->valueLabel->setText("");
+            icdw->ui->valueGreenLabel->setText("");
+            icdw->ui->valueBlueLabel->setText("");
+        }
     }
 
     xy2sky(x_cursor, y_cursor);
@@ -119,11 +128,52 @@ void IView::showCurrentMousePos(QPointF point)
     // Show the magnified area in the magnify window
     // Translate FITS pixel position in the large window to the magnify window
     icdw->magnifiedGraphicsView->mapFromScene(point.x(),point.y());
-    //magnifyScene->setSceneRect(point.x()-75, point.y()-75, 150, 150);
     icdw->magnifiedGraphicsView->setScene(icdw->magnifiedScene);
     icdw->magnifiedGraphicsView->show();
 }
 
+void IView::collectLocalStatisticsSample(QPointF point)
+{
+    if (!statdw->isVisible()) return;
+
+    // Pixel index in the 2D image
+    long x = point.x();
+    long y = naxis2 - point.y();
+
+    int s = statdw->statWidth;
+
+    int imin = x-s;
+    int imax = x+s;
+    int jmin = y-s;
+    int jmax = y+s;
+    if (imin < 0) imin = 0;
+    if (imax >= naxis1) imax = naxis1-1;
+    if (jmin < 0) jmin = 0;
+    if (jmax >= naxis2) jmax = naxis2-1;
+    if (displayMode == "FITSmonochrome" || displayMode == "MEMview") {
+        QVector<float> sample;
+        for (int j = jmin; j<=jmax; ++j) {
+            for (int i = imin; i<=imax; ++i) {
+                sample.append(fitsData[i+naxis1*j]);
+            }
+        }
+        emit statisticsSampleAvailable(sample);
+    }
+    else if (displayMode == "FITScolor") {
+        QVector<float> sampleR;
+        QVector<float> sampleG;
+        QVector<float> sampleB;
+        for (int j = jmin; j<=jmax; ++j) {
+            for (int i = imin; i<=imax; ++i) {
+                sampleR.append(fitsDataR[i+naxis1*j]);
+                sampleG.append(fitsDataG[i+naxis1*j]);
+                sampleB.append(fitsDataB[i+naxis1*j]);
+            }
+        }
+        emit statisticsSampleColorAvailable(sampleR, sampleG, sampleB);
+    }
+}
+
 void IView::adjustBrightnessContrast(QPointF point)
 {
     if (displayMode.contains("SCAMP") || displayMode == "CLEAR") return;
@@ -165,6 +215,11 @@ void IView::adjustBrightnessContrast(QPointF point)
     // Show the magnified area in the magnify window
     icdw->magnifiedGraphicsView->setScene(icdw->magnifiedScene);
     icdw->magnifiedGraphicsView->show();
+
+    emit updateNavigatorBinned(binnedPixmapItem);
+    if (wcs != nullptr && wcsInit != false) {
+        emit updateNavigatorBinnedWCS(wcs, wcsInit);
+    }
 }
 
 double IView::haversine(double x1, double y1, double x2, double y2)
@@ -227,9 +282,11 @@ void IView::drawSeparationVector(QPointF pointStart, QPointF pointEnd)
     // don't draw the vector if there is an ellipse underneath
     if (startLeftClickInsideItem) return;
 
-    QPen pen(QColor("#ffff00"));
+//    QPen pen(QColor("#ffaa00"));
+    QPen pen(QColor("#35ffff"));
     pen.setWidth(0);
-    QPen penDashed(QColor("#ffff00"));
+//    QPen penDashed(QColor("#ffaa00"));
+    QPen penDashed(QColor("#35ffff"));
     penDashed.setWidth(0);
     QVector<qreal> dashes;
     dashes << 5 << 5;
@@ -260,9 +317,19 @@ void IView::drawSeparationVector(QPointF pointStart, QPointF pointEnd)
     labelX->setFont(currentFont);
     labelY->setFont(currentFont);
     labelD->setFont(currentFont);
-    labelX->setDefaultTextColor(QColor("#ffff00"));
-    labelY->setDefaultTextColor(QColor("#ffff00"));
-    labelD->setDefaultTextColor(QColor("#ffff00"));
+    //labelX->setDefaultTextColor(QColor("#ffff00"));
+    //labelY->setDefaultTextColor(QColor("#ffff00"));
+    //labelD->setDefaultTextColor(QColor("#ffff00"));
+    // use an orange background (same as #ffaa00) for the angular distances
+    labelX->setHtml(QString("<div style='background:rgba(53, 255, 255, 50%);'>" + QString("&nbsp;") + sepXString + QString("&nbsp;") + QString("</div>") ));
+    labelY->setHtml(QString("<div style='background:rgba(53, 255, 255, 50%);'>" + QString("&nbsp;") + sepYString + QString("&nbsp;") + QString("</div>") ));
+    labelD->setHtml(QString("<div style='background:rgba(53, 255, 255, 50%);'>" + QString("&nbsp;") + sepDString + QString("&nbsp;") + QString("</div>") ));
+//    labelX->setHtml(QString("<div style='background:rgba(255, 170, 0, 100%);'>" + QString("&nbsp;") + sepXString + QString("&nbsp;") + QString("</div>") ));
+//    labelY->setHtml(QString("<div style='background:rgba(255, 170, 0, 100%);'>" + QString("&nbsp;") + sepYString + QString("&nbsp;") + QString("</div>") ));
+//    labelD->setHtml(QString("<div style='background:rgba(255, 170, 0, 100%);'>" + QString("&nbsp;") + sepDString + QString("&nbsp;") + QString("</div>") ));
+    labelX->setDefaultTextColor(QColor("#000000"));
+    labelY->setDefaultTextColor(QColor("#000000"));
+    labelD->setDefaultTextColor(QColor("#000000"));
     qreal xpos = 0.5*(x1+x2);
     qreal ypos = 0.5*(y1+y2);
     qreal x_yoffset = 0.;
@@ -342,6 +409,22 @@ void IView::clearSkyRectItems()
     }
 }
 
+void IView::clearMaskPolygonItems()
+{
+    if (!maskPolygonItems.isEmpty()) {
+        for (auto &it : maskPolygonItems ) scene->removeItem(it);
+        maskPolygonItems.clear();
+    }
+}
+
+void IView::clearMaskRectItems()
+{
+    if (!maskRectItems.isEmpty()) {
+        for (auto &it : maskRectItems ) scene->removeItem(it);
+        maskRectItems.clear();
+    }
+}
+
 void IView::clearSkyCircleItems()
 {
     if (!skyCircleItems.isEmpty()) {
@@ -426,6 +509,34 @@ void IView::middlePressResetCRPIXreceived()
     crpix2_start = wcs->crpix[1];
 }
 
+/*
+// Receiver for the event when the mouse enters the main graphics view
+void IView::mouseEnteredViewReceived()
+{
+    //    icdw->ui->navigatorStackedWidget->setCurrentIndex(1);
+    emit updateNavigatorBinned(binnedPixmapItem);
+}
+
+// Receiver for the event when the mouse leaves the main graphics view
+void IView::mouseLeftViewReceived()
+{
+    //    icdw->ui->navigatorStackedWidget->setCurrentIndex(0);
+    emit updateNavigatorBinned(binnedPixmapItem);
+}
+*/
+
+void IView::viewportChangedReceived(QRect viewport_rect)
+{
+    // the currently shown image area, in QImage coordinates
+    QRectF rect = myGraphicsView->mapToScene(viewport_rect).boundingRect();
+
+    // the matching area, in QImage coordinates of the binned overview image
+    QRect rectBinned = qimageToBinned(rect);
+
+    // send the binned rect to the binned navigator window
+    emit updateNavigatorBinnedViewport(rectBinned);
+}
+
 void IView::updateCRPIX(QPointF pointStart, QPointF pointEnd)
 {
     if (!wcsInit) return;
@@ -436,9 +547,12 @@ void IView::updateCRPIX(QPointF pointStart, QPointF pointEnd)
     // Remove items from display
     for (auto &it: refCatItems) scene->removeItem(it);
     refCatItems.clear();
+    scene->removeCrosshair();
     myGraphicsView->setScene(scene);
     myGraphicsView->show();
 
+    scene->removeCrosshair();
+
     // Recalculate CRPIX offset
     qreal dx = pointEnd.x() - pointStart.x();
     qreal dy = -(pointEnd.y() - pointStart.y());   // Image y-axis flipped in graphics view
@@ -447,6 +561,7 @@ void IView::updateCRPIX(QPointF pointStart, QPointF pointEnd)
     wcs->crpix[1] = crpix2_start + dy;
     wcs->flag = 0;    // force an update of internal wcs params.
     showReferenceCat();
+    checkFinder();
 }
 
 void IView::updateCDmatrix(double cd11, double cd12, double cd21, double cd22)
@@ -459,6 +574,7 @@ void IView::updateCDmatrix(double cd11, double cd12, double cd21, double cd22)
     // Remove items from display
     for (auto &it: refCatItems) scene->removeItem(it);
     refCatItems.clear();
+    scene->removeCrosshair();
     myGraphicsView->setScene(scene);
     myGraphicsView->show();
 
@@ -468,7 +584,13 @@ void IView::updateCDmatrix(double cd11, double cd12, double cd21, double cd22)
     wcs->cd[2] = cd21;
     wcs->cd[3] = cd22;
     wcs->flag = 0;    // force an update of internal wcs params.
+
+    icdw->cd11 = cd11;
+    icdw->cd12 = cd12;
+    icdw->cd21 = cd21;
+    icdw->cd22 = cd22;
     showReferenceCat();
+    checkFinder();
 }
 
 // Called when middle mouse button is released in wcs mode
@@ -486,8 +608,8 @@ void IView::updateCRPIXFITS()
         if (!currentMyImage->imageOnDrive) currentMyImage->writeImage();
         // Identical way to reconstruct filenames
         // TODO: uniformize
-//        qDebug() << dirName+"/"+currentMyImage->pathExtension+"/"+currentFileName;
-//        qDebug() << currentMyImage->path+"/"+currentMyImage->chipName+currentMyImage->processingStatus->statusString+".fits";
+        //        qDebug() << currentFileName;
+        //        qDebug() << currentMyImage->path+"/"+currentMyImage->chipName+currentMyImage->processingStatus->statusString+".fits";
         fitsfile *fptr = nullptr;
         fits_open_file(&fptr, (dirName+"/"+currentMyImage->pathExtension+"/"+currentFileName).toUtf8().data(), READWRITE, &status);
         fits_update_key_flt(fptr, "CRPIX1", wcs->crpix[0], -5, nullptr, &status);
@@ -551,6 +673,53 @@ void IView::drawSkyCircle(QPointF pointStart, QPointF pointEnd)
     myGraphicsView->show();
 }
 
+void IView::drawMaskingPolygon(QPointF pointStart, QPointF pointEnd)
+{
+    if (displayMode.contains("SCAMP") || displayMode == "CLEAR") return;
+    if (displayMode == "FITScolor") return;
+
+    QPen pen(QColor("#00ff00"));
+    pen.setWidth(0);
+    qreal x1 = pointStart.x();
+    qreal y1 = pointStart.y();
+    qreal x2 = pointEnd.x();
+    qreal y2 = pointEnd.y();
+
+    clearMaskRectItems();
+    QRectF rect;
+    QGraphicsRectItem *rectItem = scene->addRect(QRectF(QPoint(x1,y1), QPoint(x2,y2)));
+    rectItem->setPen(pen);
+    rectItem->setFlags(QGraphicsRectItem::ItemIsSelectable | QGraphicsRectItem::ItemIsMovable);
+    maskRectItems.append(rectItem);
+    myGraphicsView->setScene(scene);
+    myGraphicsView->show();
+}
+
+/*
+void IView::drawMaskingPolygon(QPointF pointStart, QPointF pointEnd)
+{
+    if (displayMode.contains("SCAMP") || displayMode == "CLEAR") return;
+    if (displayMode == "FITScolor") return;
+
+    QPen pen(QColor("#00ff00"));
+    pen.setWidth(0);
+    qreal x1 = pointStart.x();
+    qreal y1 = pointStart.y();
+    qreal x2 = pointEnd.x();
+    qreal y2 = pointEnd.y();
+
+    clearMaskPolygonItems();
+    QPolygonF polygon;
+    polygon << QPoint(x1,y1) << QPoint(x2,y2);
+    QGraphicsPolygonItem *polygonItem = scene->addPolygon(polygon);
+    polygonItem->setPen(pen);
+    polygonItem->setFlags(QGraphicsPolygonItem::ItemIsSelectable | QGraphicsPolygonItem::ItemIsMovable);
+    maskPolygonItems.append(polygonItem);
+    myGraphicsView->setScene(scene);
+    myGraphicsView->show();
+}
+*/
+
 void IView::appendSkyCircle()
 {
     if (displayMode.contains("SCAMP") || displayMode == "CLEAR") return;
@@ -593,6 +762,7 @@ void IView::updateSkyCircles()
     dumpSkyCircleCoordinates();
 }
 
+// invoked e.g. when dynamic range is changed
 void IView::redrawSkyCirclesAndCats()
 {
     if (displayMode.contains("SCAMP") || displayMode == "CLEAR") return;
@@ -606,6 +776,7 @@ void IView::redrawSkyCirclesAndCats()
 
     showSourceCat();
     showReferenceCat();
+    checkFinderBypass();
 }
 
 void IView::dumpSkyCircleCoordinates()
@@ -668,10 +839,10 @@ void IView::xy2sky(double x, double y, QString button)
     QString deltaDec = QString::number(world[1],'f',6);
     QString alphaHex = dec2hex(world[0]/15.).remove("+");
     QString deltaHex = dec2hex(world[1]);
-    icdw->ui->alphaDecLabel->setText("R.A. = "+alphaDec);
-    icdw->ui->alphaHexLabel->setText("R.A. = "+alphaHex);
-    icdw->ui->deltaDecLabel->setText("Dec  = "+deltaDec);
-    icdw->ui->deltaHexLabel->setText("Dec  = "+deltaHex);
+    icdw->ui->alphaDecLabel->setText(alphaDec);
+    icdw->ui->alphaHexLabel->setText(alphaHex);
+    icdw->ui->deltaDecLabel->setText(deltaDec);
+    icdw->ui->deltaHexLabel->setText(deltaHex);
 }
 
 // Unused
