@@ -28,8 +28,12 @@ If not, see https://www.gnu.org/licenses/ .
 #include "dockwidgets/ivscampdockwidget.h"
 #include "dockwidgets/ivcolordockwidget.h"
 #include "dockwidgets/ivwcsdockwidget.h"
+#include "dockwidgets/ivstatisticsdockwidget.h"
+#include "dockwidgets/ivfinderdockwidget.h"
 #include "ui_ivconfdockwidget.h"
 #include "ui_ivcolordockwidget.h"
+#include "ui_ivstatisticsdockwidget.h"
+#include "ui_ivfinderdockwidget.h"
 
 #include "../tools/tools.h"
 
@@ -56,47 +60,99 @@ If not, see https://www.gnu.org/licenses/ .
 #include <QTimer>
 #include <QModelIndex>
 
-
 void IView::setMiddleMouseMode(QString mode)
 {
     if (mode == "SkyMode") {
         ui->actionSkyMode->setChecked(true);
         ui->actionDragMode->setChecked(false);
         ui->actionWCSMode->setChecked(false);
+        ui->actionMaskingMode->setChecked(false);
         myGraphicsView->middleMouseMode = "SkyMode";
         hideWCSdockWidget();
+        icdw->ui->quitPushButton->show();
     }
     else if (mode == "DragMode") {
         ui->actionDragMode->setChecked(true);
         ui->actionSkyMode->setChecked(false);
         ui->actionWCSMode->setChecked(false);
+        ui->actionMaskingMode->setChecked(false);
         middleMouseMode = "DragMode";
         hideWCSdockWidget();
+        icdw->ui->quitPushButton->hide();
     }
     else if (mode == "WCSMode") {
         ui->actionWCSMode->setChecked(true);
         ui->actionSkyMode->setChecked(false);
         ui->actionDragMode->setChecked(false);
+        ui->actionMaskingMode->setChecked(false);
         middleMouseMode = "WCSMode";
         showWCSdockWidget();
+        icdw->ui->quitPushButton->hide();
+    }
+    else if (mode == "MaskingMode") {
+        ui->actionMaskingMode->setChecked(true);
+        ui->actionSkyMode->setChecked(false);
+        ui->actionDragMode->setChecked(false);
+        ui->actionWCSMode->setChecked(false);
+        middleMouseMode = "MaskingMode";
+        hideWCSdockWidget();
+        icdw->ui->quitPushButton->hide();
     }
 }
 
-void IView::showWCSdockWidget()
+void IView::sendWCStoWCSdockWidget()
 {
-    if (wcsdw->isVisible()) return;
+    if (!wcsInit) return;
 
     // Copy the CD matrix to the WCS dock widget and init()
     wcsdw->cd11_orig = wcs->cd[0];
     wcsdw->cd12_orig = wcs->cd[1];
     wcsdw->cd21_orig = wcs->cd[2];
     wcsdw->cd22_orig = wcs->cd[3];
+    wcsdw->crpix1_orig = wcs->crpix[0];
+    wcsdw->crpix2_orig = wcs->crpix[1];
+    wcsdw->crval1_orig = wcs->crval[0];
+    wcsdw->crval2_orig = wcs->crval[1];
     wcsdw->init();
+}
+
+void IView::showWCSdockWidget()
+{
+    if (!wcs || ! wcsInit) {
+        QMessageBox msgBox;
+        msgBox.setText("WCS not found");
+        msgBox.setInformativeText("This image does not contain any WCS information. The WCS dialog will not be shown.\n\n");
+        msgBox.addButton(tr("Ok"), QMessageBox::YesRole);
+        msgBox.exec();
+        wcsdw->setDisabled(true);
+        ui->actionWCSMode->setChecked(false);
+        return;
+    }
+
+    if (wcsdw->isVisible()) return;
+
+    sendWCStoWCSdockWidget();
 
     addDockWidget(Qt::LeftDockWidgetArea, wcsdw);
     wcsdw->setFloating(false);
     wcsdw->raise();
     wcsdw->show();
+
+    if (!ui->actionRefCat->isChecked()) {
+        // otherwise showReferenceCat() exits
+        ui->actionRefCat->setChecked(true);
+    }
+    showReferenceCat();
+    if (refCatItems.isEmpty()) {
+        QMessageBox msgBox;
+        msgBox.setText("No reference sources available");
+        msgBox.setInformativeText("You must retrieve a reference catalog first before using the WCS module.\n\n");
+        msgBox.addButton(tr("Ok"), QMessageBox::YesRole);
+        msgBox.exec();
+        wcsdw->setDisabled(true);
+        ui->actionRefCat->setChecked(false);
+        return;
+    }
 }
 
 void IView::hideWCSdockWidget()
@@ -209,10 +265,10 @@ void IView::imageListToChipName()
 void IView::setCurrentId(QString filename)
 {
     // The number of the file in the list of all images (PNG or FITS) in this directory
-    if (filterName.isEmpty() || icdw->ui->filterLineEdit->text().isEmpty()) {
+    if (filterName.isEmpty() || filterLineEdit->text().isEmpty()) {
         if (!displayMode.contains("SCAMP")) filterName = "*.fits";
         else filterName = "*.png";
-        icdw->ui->filterLineEdit->setText(filterName);
+        filterLineEdit->setText(filterName);
     }
     setImageList(filterName);
     currentId = imageList.indexOf(QFileInfo(filename).fileName());
@@ -223,12 +279,12 @@ void IView::setCurrentId(QString filename)
 
 void IView::loadImage()
 {
-    QString filter = icdw->ui->filterLineEdit->text();
+    QString filter = filterLineEdit->text();
     if (filter.isEmpty()
             || !filter.contains(".fit")
             || !filter.contains("*")) {
         filter = "*.fits";
-        icdw->ui->filterLineEdit->setText(filter);
+        filterLineEdit->setText(filter);
     }
 
     if (!QDir(dirName).exists()) dirName = QDir::homePath();
@@ -255,7 +311,7 @@ void IView::loadImage()
 
     // Update the filter string
     filterName = "*."+suffix;
-    icdw->ui->filterLineEdit->setText(filter);
+    filterLineEdit->setText(filter);
 
     if (suffix == "fits") {
         switchMode("FITSmonochrome");
@@ -300,6 +356,7 @@ void IView::clearItems() {
         for (auto &it: acceptedSkyCircleItems) scene->removeItem(it);
         acceptedSkyCircleItems.clear();
     }
+    scene->removeCrosshair();
 }
 
 void IView::setCatalogOverlaysExternally(bool sourcecatShown, bool refcatShown)
@@ -327,13 +384,13 @@ void IView::loadFITSexternal(QString fileName, QString filter)
     if (!fileName.isEmpty()) {
         switchMode("FITSmonochrome");
         filterName = filter;
-        icdw->ui->filterLineEdit->setText(filterName);
+        filterLineEdit->setText(filterName);
         loadFITS(fileName);
     }
     else {
         switchMode("CLEAR");
         filterName = filter;
-        icdw->ui->filterLineEdit->setText(filterName);
+        filterLineEdit->setText(filterName);
     }
 }
 
@@ -383,7 +440,7 @@ void IView::loadPNG(QString filename, int currentId)
     removeLastCharIf(prependPath, '/');
     QString path = get2ndLastWord(prependPath,'/')+"/"+getLastWord(prependPath,'/');
     this->setWindowTitle("iView ---   "+path+"/ ---   "+showName);
-    pageLabel->setText(" Image "+QString::number(currentId+1)+" / "+QString::number(numImages));
+    pageLabel->setText(QString::number(currentId+1)+" / "+QString::number(numImages));
 
     //  icdw->zoom2scale(zoomLevel);
     myGraphicsView->resetMatrix();
@@ -418,6 +475,7 @@ void IView::loadFITS(QString filename, int currentId, qreal scaleFactor)
         myGraphicsView->setScene(scene);
         myGraphicsView->scale(scaleFactor, scaleFactor);
         myGraphicsView->setGeometry(geometry);
+        //        myGraphicsView->centerOn(pixmapItem);
         myGraphicsView->show();
         myGraphicsView->setMinimumSize(200,200);
         myGraphicsView->setMaximumSize(10000,10000);
@@ -426,45 +484,32 @@ void IView::loadFITS(QString filename, int currentId, qreal scaleFactor)
         removeLastCharIf(prependPath, '/');
         QString path = getLastWord(prependPath,'/');
         this->setWindowTitle("iView ---   "+path+"/ ---  "+showName);
-        pageLabel->setText(" Image "+QString::number(currentId+1)+" / "+QString::number(numImages));
+        pageLabel->setText(QString::number(currentId+1)+" / "+QString::number(numImages));
 
         // Load the binned image to the navigator
         emit updateNavigatorBinned(binnedPixmapItem);       // binnedPixmapItem created in mapFITS()
+        emit updateNavigatorBinnedWCS(wcs, wcsInit);
     }
     else {
         // At end of file list, or file does not exist anymore.
         // Update file list
-        icdw->on_filterLineEdit_textChanged("dummy");
+        filterLineEdit_textChanged("dummy");
         return;
     }
 }
 
-// Receiver for the event when the mouse enters the main graphics view
-void IView::mouseEnteredViewReceived()
-{
-    icdw->ui->navigatorStackedWidget->setCurrentIndex(1);
-//    emit updateNavigatorBinned(binnedPixmapItem);
-}
-
-// Receiver for the event when the mouse leaves the main graphics view
-void IView::mouseLeftViewReceived()
-{
-    icdw->ui->navigatorStackedWidget->setCurrentIndex(0);
-//    emit updateNavigatorBinned(binnedPixmapItem);
-}
-
 void IView::loadFromRAMlist(const QModelIndex &index)
 {
     loadFromRAM(myImageList[index.row()], index.column());
 
     // Get the center image poststamp; copy() refers to the top left corner, and then width and height
-    QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(naxis1/2-icdw->navigator_nx/2,
-                                                        naxis2/2-icdw->navigator_ny/2,
-                                                        icdw->navigator_nx, icdw->navigator_ny);
+    QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(naxis1/2.-icdw->navigator_magnified_nx/2.,
+                                                        naxis2/2.-icdw->navigator_magnified_ny/2.,
+                                                        icdw->navigator_magnified_nx, icdw->navigator_magnified_ny);
     magnifiedPixmapItem = new QGraphicsPixmapItem(magnifiedPixmap);
 
     // Update the navigator magnified window with the center image poststamp
-    emit updateNavigatorMagnified(magnifiedPixmapItem, icdw->zoom2scale(zoomLevel)*magnify);
+    emit updateNavigatorMagnified(magnifiedPixmapItem, icdw->zoom2scale(zoomLevel)*magnify, 0., 0.);
 }
 
 // Receiver of Navigator currentMousePos Eevent
@@ -474,27 +519,42 @@ void IView::updateNavigatorMagnifiedReceived(QPointF point)
     qreal magnification = icdw->zoom2scale(zoomLevel)*magnify;
     if (magnification > magnify) magnification = magnify;
 
+    float dx = 0.;
+    float dy = 0.;
+
+    if (point.x() < 0 || point.x() > naxis1 || point.y() < 0 || point.y() > naxis2) {
+        emit clearMagnifiedScene();
+        return;
+    }
+
     if (displayMode == "FITSmonochrome" || displayMode == "MEMview") {
-        QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(point.x() - icdw->navigator_nx/2/magnification,
-                                                            point.y() - icdw->navigator_ny/2/magnification,
-                                                            icdw->navigator_nx/magnification, icdw->navigator_ny/magnification);
+        //        qDebug() << point.x() + 1. - icdw->navigator_magnified_nx/2./magnification << point.y() + 1. - icdw->navigator_magnified_ny/2./magnification <<
+        //                icdw->navigator_magnified_nx/magnification << icdw->navigator_magnified_ny/magnification;
+        if (point.x() + 1. - icdw->navigator_magnified_nx/2./magnification < 0) dx = point.x() + 1. - icdw->navigator_magnified_nx/2./magnification;
+        if (point.x() + 1. - icdw->navigator_magnified_nx/2./magnification >= naxis1) dx = point.x() + 1. - icdw->navigator_magnified_nx/2./magnification;
+        QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(point.x() + 1. - icdw->navigator_magnified_nx/2./magnification,
+                                                            point.y() + 1. - icdw->navigator_magnified_ny/2./magnification,
+                                                            icdw->navigator_magnified_nx/magnification, icdw->navigator_magnified_ny/magnification);
         magnifiedPixmapItem = new QGraphicsPixmapItem(magnifiedPixmap);
+        //        qDebug() << magnifiedPixmap.width() << magnifiedPixmap.height() << icdw->navigator_magnified_nx/magnification << icdw->navigator_magnified_ny/magnification;
     }
     else if (displayMode == "FITScolor") {
-        QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(point.x() - icdw->navigator_nx/2/magnification,
-                                                            point.y() - icdw->navigator_ny/2/magnification,
-                                                            icdw->navigator_nx/magnification, icdw->navigator_ny/magnification);
+        QPixmap magnifiedPixmap = pixmapItem->pixmap().copy(point.x() + 1. - icdw->navigator_magnified_nx/2/magnification,
+                                                            point.y() + 1. - icdw->navigator_magnified_ny/2/magnification,
+                                                            icdw->navigator_magnified_nx/magnification, icdw->navigator_magnified_ny/magnification);
         magnifiedPixmapItem = new QGraphicsPixmapItem(magnifiedPixmap);
     }
     else {
         // Do not emit signal for PNG mode
         return;
     }
-    emit updateNavigatorMagnified(magnifiedPixmapItem, magnification);
+    emit updateNavigatorMagnified(magnifiedPixmapItem, magnification, dx, dy);
 }
 
 void IView::loadFromRAM(MyImage *it, int indexColumn)
 {
+    scene->removeCrosshair();
+
     currentMyImage = it;
     if (indexColumn == 0 || indexColumn == 3) {
         // Load image into memory if not yet present
@@ -521,8 +581,15 @@ void IView::loadFromRAM(MyImage *it, int indexColumn)
     naxis2 = it->naxis2;
     wcs = it->wcs;
     wcsInit = it->wcsInit;
+    finderdw->dateObs = it->dateobs;
+    finderdw->geoLon = it->geolon;
+    finderdw->geoLat = it->geolat;
+
+    sendWCStoWCSdockWidget();
     this->setWindowTitle("iView --- Memory viewer : "+it->chipName);
 
+    connect(currentMyImage, &MyImage::WCSupdated, this, &IView::WCSupdatedReceived);
+
     // Get the dynamic range
     // Normal viewer mode
     if (dataIntSet) {
@@ -531,6 +598,7 @@ void IView::loadFromRAM(MyImage *it, int indexColumn)
     }
     dataInt = new unsigned char[naxis1*naxis2];
     dataIntSet = true;
+    getGlobalImageStatistics();
     // AUTO
     if (icdw->ui->minLineEdit->text().isEmpty()
             || icdw->ui->maxLineEdit->text().isEmpty()
@@ -539,8 +607,7 @@ void IView::loadFromRAM(MyImage *it, int indexColumn)
     }
     // MANUAL
     else {
-        // get background statisics (medVal and rmsVal)
-        getImageStatistics();
+        // get background statisics (median and sd)
         dynRangeMin = icdw->ui->minLineEdit->text().toFloat();
         dynRangeMax = icdw->ui->maxLineEdit->text().toFloat();
     }
@@ -557,9 +624,11 @@ void IView::loadFromRAM(MyImage *it, int indexColumn)
     myGraphicsView->setMaximumSize(10000,10000);
 
     currentMyImage = it;   // For later use, in particular when updating CRPIX1/2
+    currentFileName = currentMyImage->baseName+".fits";
 
     // Update the navigator binned window with the binned poststamp
     emit updateNavigatorBinned(binnedPixmapItem);
+    emit updateNavigatorBinnedWCS(wcs, wcsInit);
 }
 
 void IView::loadColorFITS(qreal scaleFactor)
@@ -605,15 +674,17 @@ void IView::loadColorFITS(qreal scaleFactor)
 
     // Update the navigator binned window with the binned poststamp
     emit updateNavigatorBinned(binnedPixmapItem);
+    emit updateNavigatorBinnedWCS(wcs, wcsInit);
 }
 
 bool IView::loadFITSdata(QString filename, QVector<float> &data, QString colorMode)
 {
     if (displayMode.contains("SCAMP") || displayMode == "CLEAR") {
-        qDebug() << __func__ << "Invalid mode";
         return false;
     }
 
+    scene->removeCrosshair();
+
     if (weightMode) {
         filename.replace(".fits", ".weight.fits");
     }
@@ -633,8 +704,14 @@ bool IView::loadFITSdata(QString filename, QVector<float> &data, QString colorMo
     naxis2 = currentMyImage->naxis2;
     fullheader = currentMyImage->fullheader;
     wcs = currentMyImage->wcs;
+    finderdw->dateObs = currentMyImage->dateobs;
+    finderdw->geoLon = currentMyImage->geolon;
+    finderdw->geoLat = currentMyImage->geolat;
     (void) wcsset(wcs);
     wcsInit = true;
+    sendWCStoWCSdockWidget();
+
+    connect(currentMyImage, &MyImage::WCSupdated, this, &IView::WCSupdatedReceived);
 
     // Move the data from the transient MyImage over to the class member.
     data.swap(currentMyImage->dataCurrent);        // 'fitsData' in the rest of the code
@@ -648,7 +725,9 @@ bool IView::loadFITSdata(QString filename, QVector<float> &data, QString colorMo
         }
         dataInt = new unsigned char[naxis1*naxis2];
         dataIntSet = true;
+        // determine best dynamic range for display (not yet applied)
         // AUTO
+        getGlobalImageStatistics();
         if (icdw->ui->minLineEdit->text().isEmpty()
                 || icdw->ui->maxLineEdit->text().isEmpty()
                 || icdw->ui->autocontrastPushButton->isChecked()) {
@@ -657,7 +736,6 @@ bool IView::loadFITSdata(QString filename, QVector<float> &data, QString colorMo
         // MANUAL
         else {
             // get background statisics (medVal and rmsVal)
-            getImageStatistics();
             dynRangeMin = icdw->ui->minLineEdit->text().toFloat();
             dynRangeMax = icdw->ui->maxLineEdit->text().toFloat();
         }
@@ -690,7 +768,8 @@ bool IView::loadFITSdata(QString filename, QVector<float> &data, QString colorMo
         // Loading a color view of the RGB FITS channels
         // (only executes fully once all channels have been read)
         icdw->ui->autocontrastPushButton->setChecked(true);
-        autoContrast(colorMode);
+        getGlobalImageStatistics(colorMode);
+        autoContrast();
     }
     return true;
 }
@@ -745,8 +824,9 @@ void IView::mapFITS()
     }
 
     // the scene MUST have it's final size BEFORE we add an item
-    if(scene->width() < 1 || scene->height() < 1)
-        scene->setSceneRect( 0, 0, naxis1, naxis2);
+    // WARNING: with some images (large coadds) then these images appear shifted to the right and cannot be viewed completely.
+    //    if(scene->width() < 1 || scene->height() < 1)
+    //        scene->setSceneRect( 0, 0, naxis1, naxis2);
 
     // end additional section
     //**************************************************
@@ -793,7 +873,7 @@ void IView::mapFITS()
         qDebug() << __func__ << "Invalid mode in mapFITS()";
     }
 
-    binnedPixmapItem = new QGraphicsPixmapItem(pixmapItem->pixmap().scaled(icdw->navigator_nx, icdw->navigator_ny, Qt::KeepAspectRatio, Qt::FastTransformation));
+    binnedPixmapItem = new QGraphicsPixmapItem(pixmapItem->pixmap().scaled(icdw->navigator_binned_nx, icdw->navigator_binned_ny, Qt::KeepAspectRatio, Qt::FastTransformation));
 
     /*
     if (transform) {
@@ -817,6 +897,7 @@ void IView::mapFITS()
             showG2References(true);
         }
     }
+    finderdw->bypassResolver();
 }
 
 void IView::compressDynrange(const QVector<float> &fitsdata, unsigned char *intdata, float colorCorrectionFactor)
@@ -987,9 +1068,16 @@ void IView::showReferenceCat()
             for (auto &it: refCatItems) scene->removeItem(it);
             refCatItems.clear();
         }
+        wcsdw->setDisabled(true);
+    }
+    if (!refCatItems.isEmpty()) {
+        refcatSourcesShown = true;
+        wcsdw->setEnabled(true);
+    }
+    else {
+        refcatSourcesShown = false;
+        wcsdw->setDisabled(true);
     }
-    if (!refCatItems.isEmpty()) refcatSourcesShown = true;
-    else refcatSourcesShown = false;
 
     myGraphicsView->setScene(scene);
     myGraphicsView->show();
@@ -1013,6 +1101,7 @@ void IView::showG2References(bool checked)
             if (it.contains("SDSS")) readRaDecCatalog(G2referencePathName+it, G2refCatItems, 29, width, QColor("#ee0000"));
             if (it.contains("PANSTARRS")) readRaDecCatalog(G2referencePathName+it, G2refCatItems, 24, width, QColor("#00eeee"));
             if (it.contains("APASS")) readRaDecCatalog(G2referencePathName+it, G2refCatItems, 19, width, QColor("#00ee00"));
+            if (it.contains("SKYMAPPER")) readRaDecCatalog(G2referencePathName+it, G2refCatItems, 29, width, QColor("#eeee00"));
         }
     }
     else {
@@ -1089,7 +1178,7 @@ bool IView::readRaDecCatalog(QString fileName, QList<QGraphicsRectItem*> &items,
             double dec = lineList.at(1).toDouble();
             double x = 0.;
             double y = 0.;
-            sky2xy(ra, dec, x, y);
+            sky2xyQImage(ra, dec, x, y);
             // correct for an offset introduced by where the scene draws the ellipses (or rectangles)
             point.setX(x-0.5*size);
             point.setY(y-0.5*size);
@@ -1115,7 +1204,8 @@ bool IView::readRaDecCatalog(QString fileName, QList<QGraphicsRectItem*> &items,
     }
 }
 
-void IView::sky2xy(double alpha, double delta, double &x, double &y)
+// returns coordinates in QImage system!
+void IView::sky2xyQImage(double alpha, double delta, double &x, double &y)
 {
     if (!wcsInit) return;
 
@@ -1133,6 +1223,25 @@ void IView::sky2xy(double alpha, double delta, double &x, double &y)
     y = naxis2 - pixcrd[1];
 }
 
+// returns coordinates in FITS
+void IView::sky2xyFITS(double alpha, double delta, double &x, double &y)
+{
+    if (!wcsInit) return;
+
+    double world[2];
+    double phi;
+    double theta;
+    double imgcrd[2];
+    double pixcrd[2];
+    world[0] = alpha;
+    world[1] = delta;
+    int stat[1];
+
+    wcss2p(wcs, 1, 2, world, &phi, &theta, imgcrd, pixcrd, stat);
+    x = pixcrd[0];
+    y = pixcrd[1];
+}
+
 QString IView::dec2hex(double angle)
 {
     double hh;
@@ -1156,61 +1265,57 @@ QString IView::dec2hex(double angle)
     return p+h+":"+m+":"+s;
 }
 
-void IView::getImageStatistics(QString colorMode)
+void IView::getGlobalImageStatistics(QString colorMode)
 {
     // Quasi-random sampling an array at every dim_small pixel
     int ranStep = 2./3.*naxis1 + sqrt(naxis1);
+
     QVector<double> subSample;
-    if (displayMode == "FITSmonochrome" || displayMode == "MEMview") get_array_subsample(fitsData, subSample, ranStep);
+    if (displayMode == "FITSmonochrome" || displayMode == "MEMview") {
+        get_array_subsample(fitsData, subSample, ranStep);
+        // GLOBAL statistics
+        globalMedian = medianMask_T(subSample, QVector<bool>(), "ignoreZeroes");
+        globalMean = meanMask_T(subSample, QVector<bool>());
+        globalRMS = 1.486*madMask_T(subSample, QVector<bool>(), "ignoreZeroes");
+    }
     else if (displayMode == "FITScolor") {
-        // Must have read the red and green channel already
+        QVector<double> subSampleR;
+        QVector<double> subSampleG;
+        QVector<double> subSampleB;
+        if (colorMode == "redChannel") {
+            get_array_subsample(fitsDataR, subSampleR, ranStep);
+            globalMedianR = medianMask_T(subSampleR, QVector<bool>(), "ignoreZeroes");
+        }
+        if (colorMode == "greenChannel") {
+            get_array_subsample(fitsDataG, subSampleG, ranStep);
+            globalMedianG = medianMask_T(subSampleG, QVector<bool>(), "ignoreZeroes");
+        }
         if (colorMode == "blueChannel") {
-            QVector<float> fitsDataSum(naxis1*naxis2);
-            // NOT THREADSAFE
-            // #pragma omp parallel for
-            for (long i=0; i<naxis1*naxis2; ++i) {
-                fitsDataSum[i] = fitsDataR.at(i) + fitsDataG.at(i) + fitsDataB.at(i);
+            get_array_subsample(fitsDataB, subSampleB, ranStep);
+            globalMedianB = medianMask_T(subSampleB, QVector<bool>(), "ignoreZeroes");
+            // In addition, compute overall statistics across all three bands (once the blue is read, we have all of them available)
+            get_array_subsample(fitsDataR, subSampleR, ranStep);
+            get_array_subsample(fitsDataG, subSampleG, ranStep);
+            for (long i=0; i<subSampleR.length(); ++i) {
+                subSampleR[i] += (subSampleG[i] + subSampleB[i]);
             }
-            get_array_subsample(fitsDataSum, subSample, ranStep);
+            globalMedian = medianMask_T(subSampleR, QVector<bool>(), "ignoreZeroes");
+            globalMean = meanMask_T(subSampleR, QVector<bool>());
+            globalRMS = 1.486*madMask_T(subSampleR, QVector<bool>(), "ignoreZeroes");
         }
-        else return;
     }
-    else {
-        qDebug() << "IView::getImageStatistics(): Invalid mode";
-    }
-
-    medVal = medianMask_T(subSample, QVector<bool>(), "ignoreZeroes");
-    rmsVal = madMask_T(subSample, QVector<bool>(), "ignoreZeroes");
-
-    int validDigits = 4-log(fabs(medVal))/log(10);
-    if (validDigits < 0) validDigits = 0;
-    icdw->ui->medianLabel->setText("Median = "+QString::number(medVal, 'f', validDigits));
-    icdw->ui->rmsLabel->setText("stdev  = "+QString::number(rmsVal*1.486, 'f', validDigits));
 }
 
-void IView::autoContrast(QString colorMode)
+void IView::autoContrast()
 {
-    // set medVal and rmsVal;
-    if (displayMode == "FITSmonochrome" || displayMode == "MEMview") getImageStatistics();
-    else if (displayMode == "FITScolor") {
-        // Must have read the red and green channel already
-        if (colorMode == "blueChannel" || allChannelsRead)
-            getImageStatistics(colorMode);
-        else return;
-    }
-    else {
-        qDebug() << "IView::autoContrast(): Invalid displayMode:" << displayMode;
-    }
-
-    dynRangeMin = medVal - 2.*rmsVal;
-    dynRangeMax = medVal + 10.*rmsVal;
+    dynRangeMin = globalMedian - 2.*globalRMS;
+    dynRangeMax = globalMedian + 10.*globalRMS;
     int validDigits = 3-log(fabs(dynRangeMax))/log(10);
     if (validDigits<0) validDigits = 0;
     icdw->ui->minLineEdit->setText(QString::number(dynRangeMin,'f',validDigits));
     icdw->ui->maxLineEdit->setText(QString::number(dynRangeMax,'f',validDigits));
 }
 
-
 void IView::writePreferenceSettings()
 {
     QSettings settings("IVIEW", "PREFERENCES");
@@ -1228,24 +1333,6 @@ void IView::readPreferenceSettings()
     }
 }
 
-void IView::on_actionDragMode_triggered()
-{
-    setMiddleMouseMode("DragMode");             // exclusive button group in c'tor does not work!
-    emit middleMouseModeChanged("DragMode");
-}
-
-void IView::on_actionSkyMode_triggered()
-{
-    setMiddleMouseMode("SkyMode");              // exclusive button group in c'tor does not work!
-    emit middleMouseModeChanged("SkyMode");
-}
-
-void IView::on_actionWCSMode_triggered()
-{
-    setMiddleMouseMode("WCSMode");              // exclusive button group in c'tor does not work!
-    emit middleMouseModeChanged("WCSMode");
-}
-
 void IView::solutionAcceptanceStateReceived(bool state)
 {
     emit solutionAcceptanceState(state);
@@ -1261,6 +1348,7 @@ void IView::autoContrastPushButton_toggled_receiver(bool checked)
     if (checked) {
         autoContrast();
 
+        scene->removeCrosshair();
         // Must clear all items (sky circles) on the scene, then redraw afterwards
         if (!skyCircleItems.isEmpty()) skyCircleItems.clear();
         if (!acceptedSkyCircleItems.isEmpty()) {
@@ -1272,6 +1360,9 @@ void IView::autoContrastPushButton_toggled_receiver(bool checked)
         myGraphicsView->setScene(scene);
         myGraphicsView->show();
         redrawSkyCirclesAndCats();
+
+        emit updateNavigatorBinned(binnedPixmapItem);
+        emit updateNavigatorBinnedWCS(wcs, wcsInit);
     }
 }
 
@@ -1288,6 +1379,8 @@ void IView::replotCatalogAfterZoom()
         ui->actionRefCat->setChecked(true);
         showReferenceCat();
     }
+    scene->zoomScale = icdw->zoom2scale(zoomLevel);
+    finderdw->bypassResolver();
 }
 
 void IView::zoomFitPushButton_clicked_receiver(bool checked)
@@ -1299,9 +1392,9 @@ void IView::zoomFitPushButton_clicked_receiver(bool checked)
         myGraphicsView->fitInView(scene->items(Qt::AscendingOrder).at(0), Qt::KeepAspectRatio);
         double scale = myGraphicsView->transform().m11();
         QString scaleFactor;
-        if (scale > 1.) scaleFactor = "Zoom level: "+QString::number(scale,'f',2)+":1";
-        else scaleFactor = "Zoom level: 1:"+QString::number(1./scale,'f',2);
-        icdw->ui->zoomLabel->setText(scaleFactor);
+        if (scale > 1.) scaleFactor = QString::number(scale,'f',2)+":1";
+        else scaleFactor = "1:"+QString::number(1./scale,'f',2);
+        //        icdw->ui->zoomValueLabel->setText(scaleFactor);
     }
 
     replotCatalogAfterZoom();
@@ -1361,3 +1454,60 @@ void IView::colorFactorChanged_receiver(QString redFactor, QString blueFactor)
     // Apply color factors:
     mapFITS();
 }
+
+void IView::filterLineEdit_textChanged(const QString &arg1)
+{
+    QString filter = arg1;
+
+    if (filter.isEmpty()
+            || !filter.contains(".fits")
+            || !filter.contains("*")) {
+        filter = "*.fits";
+        filterLineEdit->setText(filter);
+    }
+
+    setImageList(filter);
+    numImages = imageList.length();
+
+    pageLabel->setText("? / "+QString::number(numImages));
+
+    // Rewind
+    //    iview->startAction_triggered();
+}
+
+void IView::fovCenterChangedReceiver(QPointF newCenter)
+{
+    QPointF center = binnedToQimage(newCenter);
+    //    qDebug() << newCenter << center;
+    myGraphicsView->centerOn(center);
+}
+
+void IView::targetResolvedReceived(QString alphaStr, QString deltaStr)
+{
+    if (alphaStr.contains(":")) alphaStr = hmsToDecimal(alphaStr);
+    if (deltaStr.contains(":")) deltaStr = dmsToDecimal(deltaStr);
+
+    double x = 0.;
+    double y = 0.;
+    sky2xyQImage(alphaStr.toDouble(), deltaStr.toDouble(), x, y);
+    if (x > 0. && x < naxis1-1 && y > 0. && y < naxis2-1) {
+        myGraphicsView->centerOn(QPointF(x,y));
+        scene->addCrosshair(x,y);
+    }
+    else {
+/*
+ *         if (sender() == finderdw->ui->targetresolverToolButton) {
+            finderdw->ui->targetNameNonsiderealLineEdit->setText("outside f.o.v");
+        }
+        if (sender() == finderdw->ui->MPCresolverToolButton) {
+            finderdw->ui->targetNameSiderealLineEdit->setText("outside f.o.v");
+        }
+*/
+    }
+}
+
+void IView::WCSupdatedReceived()
+{
+    sendWCStoWCSdockWidget();
+    icdw->receiveWCS(wcs, wcsInit);
+}
