@@ -131,6 +131,7 @@ MainWindow::MainWindow(QString pid, QWidget *parent) :
 
     // NOTE: setting up the controller may take a while (when creating object masks)
     controller = new Controller(&instData, statusOld, cdw, monitor, this);
+
     // LineEdit changes update controller
     for (auto &it : status.listDataDirs) {
         //        connect(it, &QLineEdit::textChanged, controller, &Controller::updateSingle);
@@ -140,8 +141,8 @@ MainWindow::MainWindow(QString pid, QWidget *parent) :
     }
 
     // Initial configuration of MainWindow
-    initGUI();
 
+    initGUI();
     // Repaint background of previously executed tasks.
     // initGUI loads the style sheet, thus defaulting to backgrounds
     status.history2checkbox();
@@ -209,6 +210,10 @@ MainWindow::MainWindow(QString pid, QWidget *parent) :
     startProgressBars(); // must (or should) be done after settings are read and controller is live
 
     estimateBinningFactor();
+
+    setHomeDir();  // mostly first time launch, only
+
+    updateExcludedDetectors(cdw->ui->excludeDetectorsLineEdit->text());
 }
 
 void MainWindow::closeEvent(QCloseEvent *event)
@@ -692,11 +697,6 @@ void MainWindow::updateDiskspaceWarning(int newLimit)
     datadiskspace_warned = false;
 }
 
-void MainWindow::updateServer(QString server)
-{
-    downloadServer = server;
-}
-
 void MainWindow::updateNumcpu(int cpu)
 {
     numCPU = cpu;
@@ -896,7 +896,7 @@ void MainWindow::resetInstrumentData()
     instData.shortName = "";
     instData.nameFullPath = "";
     instData.obslat = 0.;
-    instData.obslong = 0.;
+    instData.obslon = 0.;
     instData.bayer = "";
     instData.flip = "";
     instData.type = "OPT";
@@ -951,7 +951,7 @@ void MainWindow::initInstrumentData(QString instrumentNameFullPath)
             bayerFound = true;
         }
         if (line.contains("OBSLAT=")) instData.obslat = line.split("=")[1].toFloat();
-        if (line.contains("OBSLONG=")) instData.obslong = line.split("=")[1].toFloat();
+        if (line.contains("OBSLONG=")) instData.obslon = line.split("=")[1].toFloat();
         if (line.contains("PIXSCALE=")) instData.pixscale = line.split("=")[1].toFloat();
         //        if (line.contains("GAIN=")) instData.gain = line.split("=")[1].toFloat();
 
@@ -1236,6 +1236,7 @@ void MainWindow::on_setupInstrumentComboBox_clicked()
         ui->BACconfigureToolButton->show();
         cdw->ui->overscanCheckBox->setEnabled(true);
         cdw->ui->overscanMethodComboBox->setEnabled(true);
+        if (cdw->ui->saturationLineEdit->text().isEmpty()) cdw->ui->saturationLineEdit->setText("65535");
     }
 
     cdw->ui->BIPSpinBox->setValue(estimateBinningFactor());
@@ -1407,7 +1408,6 @@ void MainWindow::loadPreferences()
     connect(preferences, &Preferences::fontSizeChanged, this, &MainWindow::updateFontSize);
     connect(preferences, &Preferences::fontChanged, this, &MainWindow::updateFont);
     connect(preferences, &Preferences::diskspacewarnChanged, this, &MainWindow::updateDiskspaceWarning);
-    connect(preferences, &Preferences::serverChanged, this, &MainWindow::updateServer);
     connect(preferences, &Preferences::numcpuChanged, this, &MainWindow::updateNumcpu);
     connect(preferences, &Preferences::memoryUsageChanged, controller, &Controller::updateMemoryPreference);
     connect(preferences, &Preferences::switchProcessMonitorChanged, this, &MainWindow::updateSwitchProcessMonitorPreference);
@@ -1435,6 +1435,10 @@ void MainWindow::statusChangedReceived(QString newStatus)
 
 void MainWindow::updateExcludedDetectors(QString badDetectors)
 {
+    if (doingInitialLaunch && readingSettings) return;
+
+    if (controller) controller->successProcessing = true;         // function called at launch when controller does not yet exist
+
     instData.badChips.clear();
     QStringList chipStringList = badDetectors.replace(","," ").simplified().split(" ");
     if (!badDetectors.isEmpty()) {
@@ -1442,6 +1446,20 @@ void MainWindow::updateExcludedDetectors(QString badDetectors)
     }
     instData.numUsedChips = instData.numChips - instData.badChips.length();
 
+    instData.goodChips.clear();
+    for (int chip=0; chip<instData.numChips; ++chip) {
+        if (!instData.badChips.contains(chip)) instData.goodChips.append(chip);
+    }
+    if (instData.goodChips.isEmpty()) {
+        if (controller) controller->successProcessing = false;
+        QMessageBox::critical(this, tr("THELI"),
+                              tr("No valid detectors remain after filtering bad detectors. Review the list of user-defined unusable detectors."),
+                              QMessageBox::Ok);
+    }
+    else {
+        instData.validChip = instData.goodChips[0];
+    }
+
     // Map the chip numbers to the number in which they appear in order (e.g. in .scamp catalogs)
     int countGoodChip = 0;
     for (int chip=0; chip<instData.numChips; ++chip) {
@@ -1560,7 +1578,7 @@ void MainWindow::loadIView()
 
     if (!QDir(dirName).exists()) dirName = QDir::homePath();
 
-    IView *iView = new IView("FITSmonochrome", dirName, "*.fits *.fit *.FITS *.FIT", this);
+    IView *iView = new IView("FITSmonochrome", dirName, "*.fits", this);
     iView->show();
 }
 
@@ -1637,7 +1655,7 @@ void MainWindow::resetParameters()
     else {
         QMessageBox msgBox;
         msgBox.setModal(true);
-        msgBox.setInformativeText(tr("The parameters in the current project are about to be reset. The project contains\n") +
+        msgBox.setInformativeText(tr("The parameters in the current project are about to be reset.\n") +
                                   tr(" All processing parameters will be reset to their default values. Processing will continue with the FITS files found on disk.\n"));
         QAbstractButton *pButtonOK = msgBox.addButton(tr("OK"), QMessageBox::YesRole);
         QAbstractButton *pButtonCancel = msgBox.addButton(tr("Cancel"), QMessageBox::YesRole);
@@ -1906,3 +1924,17 @@ void MainWindow::on_setupProjectLineEdit_textChanged(const QString &arg1)
 {
     this->setWindowTitle("THELI "+GUIVERSION+"      Project: "+arg1);
 }
+
+// Used e.g. when user starts the very first time and nothing has been defined yet
+void MainWindow::setHomeDir()
+{
+    if (ui->setupBiasLineEdit->text().isEmpty()
+            && ui->setupDarkLineEdit->text().isEmpty()
+            && ui->setupFlatoffLineEdit->text().isEmpty()
+            && ui->setupFlatLineEdit->text().isEmpty()
+            && ui->setupScienceLineEdit->text().isEmpty()
+            && ui->setupSkyLineEdit->text().isEmpty()
+            && ui->setupStandardLineEdit->text().isEmpty()
+            && ui->setupMainLineEdit->text().isEmpty())
+        ui->setupMainLineEdit->setText(QDir::homePath());
+}
