@@ -40,12 +40,12 @@ void MyImage::readFILTER(QString loadFileName)
     fits_read_key_str(fptr, "FILTER", filterchar, NULL, &status);
     fits_close_file(fptr, &status);
     QString filterstring(filterchar);
-    printCfitsioError("MyImage::readFILTER()", status);
+    printCfitsioError(__func__, status);
 
     filter = filterstring.simplified();
 }
 
-// Extract the NAXIS1, NAXIS2, CRPIX1, CRPIX2, SKYVALUE keywords from a yet unopened FITS file for swarpfilter()
+// Extract the NAXIS1, NAXIS2, CRPIX1, CRPIX2, SKYVALUE and FLXSCALE keywords from a yet unopened (resampled) FITS file for swarpfilter()
 bool MyImage::informSwarpfilter(long &naxis1, long &naxis2, double &crpix1, double &crpix2, double &sky, double &fluxscale)
 {
     QString loadFileName = path + "/" + name;
@@ -56,10 +56,25 @@ bool MyImage::informSwarpfilter(long &naxis1, long &naxis2, double &crpix1, doub
     fits_read_key_dbl(fptr, "CRPIX2", &crpix2, NULL, &status);
     fits_read_key_lng(fptr, "NAXIS1", &naxis1, NULL, &status);
     fits_read_key_lng(fptr, "NAXIS2", &naxis2, NULL, &status);
-    fits_read_key_dbl(fptr, "SKYVALUE", &sky, NULL, &status);
     fits_read_key_dbl(fptr, "FLXSCALE", &fluxscale, NULL, &status);
+
+    // get the sky value from the MyImage if not present in the header of the FITS file
+    // (in case the user did not run the calibration step)
+    int preSky = status;
+    fits_read_key_dbl(fptr, "SKYVALUE", &sky, NULL, &status);
+    int postSky = status;
+    if (!preSky && postSky) {
+        sky = skyValue;
+        if (sky != 0.) status = 0;
+        else status = 1;
+    }
     fits_close_file(fptr, &status);
-    printCfitsioError("MyImage::informSwarpfilter()", status);
+    printCfitsioError(__func__, status);
+    if (fluxscale == 0.) {
+        emit messageAvailable(name + ": FLXSCALE = 0 in resampled FITS header. Relative photometry failed or not propagated correctly during scamp run", "error");
+        emit critical();
+        return false;
+    }
     if (status) return false;
     else return true;
 }
@@ -136,6 +151,7 @@ void MyImage::readData(fitsfile **fptr, int *status)
             // else data[i] = buffer[i];
             dataCurrent[i] = buffer[i];
         }
+        imageInMemory = true;
     }
     dataCurrent.squeeze(); // shed excess memory
 
@@ -171,6 +187,14 @@ void MyImage::readDataWeight(fitsfile **fptr, int *status)
     }
     dataWeight.squeeze(); // shed excess memory
 
+    // if the weight image is loaded before the science data,
+    // e.g. the GUI is started and the user goes into the memory viewer to load the weight directly,
+    // then naxis1/2 is not defined, leading to a crash
+    if (naxis1 == 0 && naxis2 == 0) {
+        naxis1 = nax1;
+        naxis2 = nax2;
+    }
+
     delete [] buffer;
     buffer = nullptr;
 }
@@ -193,7 +217,7 @@ bool MyImage::loadData(QString loadFileName)
     cornersToRaDec();
     fits_close_file(fptr, &status);
 
-    printCfitsioError("MyImage::loadData()", status);
+    printCfitsioError(__func__, status);
 
     if (!status) headerInfoProvided = true;
     else headerInfoProvided = false;
@@ -220,7 +244,7 @@ bool MyImage::loadDataThreadSafe(QString loadFileName)
         cornersToRaDec();
         fits_close_file(fptr, &status);
 
-        printCfitsioError("MyImage::loadData()", status);
+        printCfitsioError(__func__, status);
 
         if (!status) headerInfoProvided = true;
         else headerInfoProvided = false;
@@ -273,7 +297,7 @@ void MyImage::initWCS()
         return;
     }
     wcsInit = true;
-    plateScale = sqrt(wcs->cd[0] * wcs->cd[0] + wcs->cd[2] * wcs->cd[2]) * 3600.;       // in arcsec
+    plateScale = sqrt(wcs->cd[0] * wcs->cd[0] + wcs->cd[2] * wcs->cd[2]) * 3600.;       // in arcsec; technically, this is the increment in the x-axis
     if (plateScale == 0.) plateScale = 1.0;
 
     if (*verbosity > 2) {
@@ -306,7 +330,7 @@ void MyImage::loadHeader(QString loadFileName)
 
     fits_close_file(fptr, &status);
 
-    printCfitsioError("MyImage::loadHeader()", status);
+    printCfitsioError(__func__, status);
 
     if (!status) headerInfoProvided = true;
     else headerInfoProvided = false;
@@ -325,7 +349,7 @@ void MyImage::getMJD()
     fits_read_key_dbl(fptr, "MJD-OBS", &mjdobs, NULL, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("getMJD()", status);
+    printCfitsioError(__func__, status);
     if (status) hasMJDread = false;
     else hasMJDread = true;
 }
@@ -345,6 +369,8 @@ void MyImage::initTHELIheader(int *status)
         extractKeywordInt(it, "BITPIX", bitpix);
         extractKeywordFloat(it, "SKYVALUE", skyValue);
         extractKeywordFloat(it, "AIRMASS", airmass);
+        extractKeywordFloat(it, "GEOLON", geolon);
+        extractKeywordFloat(it, "GEOLAT", geolat);
         extractKeywordFloat(it, "FWHM", fwhm);
         extractKeywordFloat(it, "FWHMEST", fwhm_est);
         extractKeywordFloat(it, "GAIN", gain);
@@ -503,6 +529,6 @@ void MyImage::loadDataSection(long xmin, long xmax, long ymin, long ymax, float
     fits_read_subset(fptr, TFLOAT, fpixel, lpixel, strides, &nullval, dataSect, &anynull, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("MyImage::stayWithinBounds()", status);
+    printCfitsioError(__func__, status);
 }
 
