@@ -214,13 +214,10 @@ void MyImage::readImage(bool determineMode)
     else {
         if (loadData()) {
             getMode(determineMode);
-            imageInMemory = true;
             imageOnDrive = true;
             successProcessing = true;
-            headerInfoProvided = true;
             updateProcInfo("Loaded");
             if (*verbosity > 2) emit messageAvailable(chipName+" : Loaded", "image");
-
         }
         else {
             emit messageAvailable("MyImage::readImage(): Could not load " + baseName + ".fits", "error");
@@ -246,10 +243,8 @@ void MyImage::readImageThreadSafe(bool determineMode)
     else {
         if (loadDataThreadSafe()) {
             getMode(determineMode);
-            imageInMemory = true;
             imageOnDrive = true;
             successProcessing = true;
-            headerInfoProvided = true;
             updateProcInfo("Loaded");
             if (*verbosity > 2) emit messageAvailable(chipName+" : Loaded", "image");
 
@@ -278,10 +273,8 @@ void MyImage::readImage(QString loadFileName)
     }
     else {
         if (loadData(loadFileName)) {
-            imageInMemory = true;
             imageOnDrive = true;
             successProcessing = true;
-            headerInfoProvided = true;
             updateProcInfo("Loaded");
             if (*verbosity > 2) emit messageAvailable(chipName+" : Loaded", "image");
         }
@@ -306,11 +299,9 @@ void MyImage::readImageBackupL1Launch()
         bool determineMode = true;
         getMode(determineMode);
         dataBackupL1 = dataCurrent;
-        imageInMemory = true;
         backupL1OnDrive = true;
         backupL1InMemory = true;
         successProcessing = true;
-        headerInfoProvided = true;
         updateProcInfo("Loaded");
         if (*verbosity > 2) emit messageAvailable(baseNameBackupL1+" : Loaded", "image");
     }
@@ -355,7 +346,7 @@ void MyImage::readImageBackupL1()
         fits_open_file(&fptr, backupName.toUtf8().data(), READONLY, &status);
         fits_read_img(fptr, TFLOAT, fpixel, nelements, &nullval, buffer, &anynull, &status);
         fits_close_file(fptr, &status);
-        printCfitsioError("readImageBackupL1()", status);
+        printCfitsioError(__func__, status);
 
         dataBackupL1.resize(nelements);
         for (long i=0; i<nelements; ++i) {
@@ -403,9 +394,10 @@ bool MyImage::scanAstromHeader(int chip, QString mode)
     while(!in.atEnd()) {
         QString line = in.readLine().simplified();
         if (line.isEmpty()) continue;
-        QStringList list = line.split(" ");
-        if (list.length() < 3) continue;
-        if (line.contains("CRVAL1")) wcs->crval[0] = list[2].toDouble();
+        QStringList list = line.split("=");
+        if (list.length() < 2) continue;
+        QString value = list.at(1).simplified().split(" ").at(0);
+        /*if (line.contains("CRVAL1")) wcs->crval[0] = list[2].toDouble();
         if (line.contains("CRVAL2")) wcs->crval[1] = list[2].toDouble();
         if (line.contains("CRPIX1")) wcs->crpix[0] = list[2].toFloat();
         if (line.contains("CRPIX2")) wcs->crpix[1] = list[2].toFloat();
@@ -415,8 +407,18 @@ bool MyImage::scanAstromHeader(int chip, QString mode)
         if (line.contains("CD2_2")) wcs->cd[3] = list[2].toDouble();
         if (line.contains("FLXSCALE")) FLXSCALE = list[2].toFloat();
         if (line.contains("RZP")) RZP = list[2].toFloat();
+        */
+        if (line.contains("CRVAL1")) wcs->crval[0] = value.toDouble();
+        if (line.contains("CRVAL2")) wcs->crval[1] = value.toDouble();
+        if (line.contains("CRPIX1")) wcs->crpix[0] = value.toFloat();
+        if (line.contains("CRPIX2")) wcs->crpix[1] = value.toFloat();
+        if (line.contains("CD1_1")) wcs->cd[0] = value.toDouble();
+        if (line.contains("CD1_2")) wcs->cd[1] = value.toDouble();
+        if (line.contains("CD2_1")) wcs->cd[2] = value.toDouble();
+        if (line.contains("CD2_2")) wcs->cd[3] = value.toDouble();
+        if (line.contains("FLXSCALE")) FLXSCALE = value.toFloat();
+        if (line.contains("RZP")) RZP = value.toFloat();
     }
-
     file.close();
 
     // Do not update the WCS matrix if it is significantly flawed
@@ -424,6 +426,7 @@ bool MyImage::scanAstromHeader(int chip, QString mode)
         wcs->flag = 0;  // Trigger recomputation
         cornersToRaDec();
         updateCRVALCRPIXCDinHeader();
+        emit WCSupdated();
         return true;
     }
     else {
@@ -487,7 +490,7 @@ void MyImage::backupOrigHeader(int chip)
     }
     if (status == END_OF_FILE) status = 0;
     else {
-        printCfitsioError("backupOrigHeader()", status);
+        printCfitsioError(__func__, status);
     }
     fits_close_file(fptr, &status);
     file.close();
@@ -614,6 +617,11 @@ void MyImage::updateHeaderValue(QString keyName, QString keyValue)
     }
 }
 
+void MyImage::updateHeaderSaturation()
+{
+    updateHeaderValue("SATURATE", saturationValue);
+}
+
 void MyImage::updateHeaderValueInFITS(QString keyName, QString keyValue)
 {
     if (!successProcessing) return;
@@ -627,7 +635,7 @@ void MyImage::updateHeaderValueInFITS(QString keyName, QString keyValue)
     fits_open_file(&fptr, (path+"/"+name).toUtf8().data(), READWRITE, &status);
     fits_update_key_str(fptr, keyName.toUtf8().data(), keyValue.toUtf8().data(), nullptr, &status);
     fits_close_file(fptr, &status);
-    printCfitsioError("updateHeaderValueInFITS", status);
+    printCfitsioError(__func__, status);
 }
 
 double MyImage::getPlateScale()
@@ -650,6 +658,7 @@ void MyImage::getMode(bool determineMode)
     // Get the mode only if requested, and measure it only if it hasn't been measured already
     // (in which case it is available as the SKYVALUE header keyword)
     if (determineMode && !modeDetermined) {
+        if (!imageInMemory) loadData();
         skyValue = modeMask(dataCurrent, "stable", globalMask)[0];
         modeDetermined = true;
         QString skyvalue = "SKYVALUE= "+QString::number(skyValue);
@@ -723,6 +732,7 @@ void MyImage::subtractBias(const MyImage *biasImage, QString dataType)
     }
 
     saturationValue -= biasImage->skyValue;
+    updateHeaderValue("SATURATE", saturationValue);
 
     QString mode = "";
     if (dataType == "BIAS") mode = "bias subtracted";
@@ -796,6 +806,7 @@ void MyImage::divideFlat(const MyImage *flatImage)
     }
 
     saturationValue /= flatImage->gainNormalization;
+    updateHeaderValue("SATURATE", saturationValue);
 
     if (*verbosity > 1) emit messageAvailable(chipName + " : Flat fielded", "image");
     successProcessing = true;
@@ -820,6 +831,7 @@ void MyImage::applyBackgroundModel(const MyImage *backgroundImage, QString mode,
         }
 
         saturationValue -= backgroundImage->skyValue * rescale;
+        updateHeaderValue("SATURATE", saturationValue);
 
         QString img = " IMG = "+QString::number(skyValue, 'f', 3) + ";";
         QString back = " BACK = "+QString::number(backgroundImage->skyValue, 'f', 3) + ";";
@@ -839,6 +851,7 @@ void MyImage::applyBackgroundModel(const MyImage *backgroundImage, QString mode,
         }
 
         saturationValue /= backgroundImage->skyValue;
+        updateHeaderValue("SATURATE", saturationValue);
 
         QString img = " IMG = "+QString::number(skyValue, 'f', 3) + ";";
         QString back = " BACK = "+QString::number(backgroundImage->skyValue, 'f', 3) + ";";
@@ -1019,16 +1032,18 @@ void MyImage::removeSourceCatalogs()
 
 // This routine is used when reading an external image
 // (and possibly its existing weight in the same directory, such as a pair of coadd.fits and coadd.weight.fits)
+// So far only used for creating astrometric reference catalogs
 void MyImage::setupCoaddMode()
 {
-    readImage();
+    readImage(path+"/"+name);
 
     // Setup an empty dummy globalMask
     //    globalMask = QVector<bool>();
     globalMaskAvailable = false;
 
     // Load a matching weight image, if one exists
-    weightName.append(".fits");
+    // weightName.append(".fits");
+    weightName = baseName + ".weight.fits";
 
     QFile file(path+"/"+weightName);
     if (!file.exists()) return;
@@ -1077,11 +1092,12 @@ void MyImage::updateZeroOrderOnDrive(QString updateMode)
     fits_update_key_dbl(fptr, "CD2_2", wcs->cd[3], 9, nullptr, &status);
     if (std::isnan(RZP)) {
         RZP = 0.;
-        FLXSCALE = 0.;
-        emit messageAvailable(chipName + " : Scamp could not determine the relative zeropoint. Set to 0!", "warning");
+        FLXSCALE = 0.0;
+        emit messageAvailable(chipName + " : Scamp could not determine the relative zeropoint. Set to 0!", "error");
+        emit critical();
     }
-    fits_update_key_flt(fptr, "RZP", RZP, 3, nullptr, &status);
-    fits_update_key_flt(fptr, "FLXSCALE", FLXSCALE, 3, nullptr, &status);
+    fits_update_key_flt(fptr, "RZP", RZP, 5, nullptr, &status);
+    fits_update_key_flt(fptr, "FLXSCALE", FLXSCALE, 5, nullptr, &status);
     if (updateMode == "restore") {
         fits_update_key_str(fptr, "ZEROHEAD", "NO", "Astrometric header update", &status); // reset the update flag
     }
@@ -1091,7 +1107,7 @@ void MyImage::updateZeroOrderOnDrive(QString updateMode)
     }
 
     fits_close_file(fptr, &status);
-    printCfitsioError("updateZeroOrderOnDrive()", status);
+    printCfitsioError(__func__, status);
 }
 
 /*
@@ -1165,7 +1181,7 @@ void MyImage::updateCRVALinHeaderOnDrive()
     fits_update_key_dbl(fptr, "CRVAL1", wcs->crval[0], 6, nullptr, &status);
     fits_update_key_dbl(fptr, "CRVAL2", wcs->crval[1], 6, nullptr, &status);
     fits_close_file(fptr, &status);
-    printCfitsioError("updateCRVALinHeaderOnDrive()", status);
+    printCfitsioError(__func__, status);
 }
 
 void MyImage::updateCRVALCDinHeaderOnDrive()
@@ -1185,7 +1201,7 @@ void MyImage::updateCRVALCDinHeaderOnDrive()
     fits_update_key_flt(fptr, "CD2_1", wcs->cd[2], 6, nullptr, &status);
     fits_update_key_flt(fptr, "CD2_2", wcs->cd[3], 6, nullptr, &status);
     fits_close_file(fptr, &status);
-    printCfitsioError("updateCRVALCDinHeaderOnDrive()", status);
+    printCfitsioError(__func__, status);
 }
 
 void MyImage::updateInactivePath()
@@ -1456,7 +1472,7 @@ void MyImage::mergeObjectWithGlobalMask()
     else {
         long i=0;
         for (auto &pixel : objectMask) {
-            pixel *= globalMask.at(i);
+            pixel = pixel && globalMask.at(i);
             ++i;
         }
     }
