@@ -50,13 +50,6 @@ void MyImage::resetObjectMasking()
     objectMask.squeeze();
 }
 
-void MyImage::updateSaturation(QString saturation)
-{
-    if (!saturation.isEmpty()) {
-        saturationValue = saturation.toFloat();
-    }
-}
-
 void MyImage::segmentImage(const QString DTstring, const QString DMINstring, const bool convolution, const bool writeSegImage)
 {
     if (*verbosity > 1) emit messageAvailable(chipName + " : Detecting objects ... ", "image");
@@ -78,7 +71,7 @@ void MyImage::segmentImage(const QString DTstring, const QString DMINstring, con
     // Noise clipping
     // if this doesn't work, then make controller a member of each myimage
     emit setMemoryLock(true);
-    dataSegmentation.resize(naxis1*naxis2);
+    dataSegmentation.resize(naxis1*naxis2);         // this crashes vor very large images (e.g. coadded omegacam data); likely internal Qvector limitation
     dataMeasure.clear();   // Must clear (if we run the detection twice, e.g. for sky modeling)
   //  dataMeasure.reserve(naxis1*naxis2);
     dataMeasure.resize(naxis1*naxis2);
@@ -650,6 +643,10 @@ void MyImage::writeCatalog(QString minFWHM_string, QString maxFlag_string)
         }
         file.close();
     }
+    else {
+        emit messageAvailable("Could not create IView source catalog: "+path+"/cat/iview/"+chipName+".iview", "error");
+        emit critical();
+    }
 
     // Write anet catalog
     char x[100] = "X";
@@ -697,7 +694,7 @@ void MyImage::writeCatalog(QString minFWHM_string, QString maxFlag_string)
     fits_write_col(fptr, TFLOAT, 3, firstrow, firstelem, nrows, mag_arr, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("MyImage::writeCatalog()", status);
+    printCfitsioError(__func__, status);
 }
 
 void MyImage::makeXcorrData()
@@ -861,5 +858,5 @@ void MyImage::appendToScampCatalogInternal(fitsfile *fptr, QString minFWHM_strin
 
     messageAvailable(rootName + " : " + QString::number(nrows) + " sources after filtering..." +detStatus, level);
 
-    printCfitsioError("MyImage::appendToScampCatalogInternal())", status);
+    printCfitsioError(__func__, status);
 }
