@@ -40,6 +40,8 @@ If not, see https://www.gnu.org/licenses/ .
 #include <QTimer>
 #include <QProgressBar>
 
+#include <unistd.h>
+
 Controller::Controller(const instrumentDataType *instrumentData, QString statusold, ConfDockWidget *confDockWidget,
                        Monitor *processMonitor, MainWindow *parent) :
     QMainWindow(parent),
@@ -98,6 +100,7 @@ Controller::Controller(const instrumentDataType *instrumentData, QString statuso
     // Done at top level, so it is not repeated many times by the Splitter class
     populateHeaderDictionary();
     populateFilterDictionary();
+    populateInstrumentDictionary();
 
     // Daisy-chaining scamp, and the coadditions
     connect(this, &Controller::showScampCheckPlots, this, &Controller::showScampCheckPlotsReceived);
@@ -505,7 +508,7 @@ void Controller::updateAll()
     mask->instData = instData;
     mask->initMasks();
 
-    // Fro GROND (and in the future, other multi-channel instruments)
+    // For GROND (and other multi-channel instruments)
     provideAlternativeMask();
 
     // instData member is a pointer and does not need updating when a new instrument is selected (true? must check!)
@@ -515,15 +518,15 @@ void Controller::updateAll()
 // so far called in MainGUIWorker::runTask(); should be done better by signal emission
 void Controller::loadPreferences()
 {
+    // Start with max number of CPUs (updated with user preference below)
     QSettings settings("THELI", "PREFERENCES");
-    maxCPU = settings.value("prefCPUSpinBox").toInt();
-    maxThreadsIO = settings.value("prefIOthreadsSpinBox").toInt();
-    useGPU = settings.value("prefGPUCheckBox").toBool();
+    maxCPU = settings.value("prefCPUSpinBox", QThread::idealThreadCount()).toInt();
+    useGPU = settings.value("prefGPUCheckBox", false).toBool();
     maxRAM = 0.75 * settings.value("prefMemorySpinBox").toInt();                   // 75% of RAM
-    verbosity = settings.value("prefVerbosityComboBox").toInt();
-    if (settings.value("prefIntermediateDataComboBox") == "If necessary") alwaysStoreData = false;
+    verbosity = settings.value("prefVerbosityComboBox", 1).toInt();
+    if (settings.value("prefIntermediateDataComboBox", "If necessary") == "If necessary") alwaysStoreData = false;
     else alwaysStoreData = true;
-    minimizeMemoryUsage = settings.value("prefMemoryCheckBox").toBool();
+    minimizeMemoryUsage = settings.value("prefMemoryCheckBox", false).toBool();
 
     availableThreads = maxCPU;
 
@@ -545,7 +548,7 @@ void Controller::loadPreferences()
     // (internal threads in Data class)
     if (maxCPU > instData->numUsedChips) {
         //        maxExternalThreads = instData->numUsedChips;      // NO! The loops skipped for a bad chip will just wait until another CPU becomes available.
-        maxExternalThreads = instData->numChips;            // keep full number of CPUs instead
+        maxExternalThreads = instData->numChips;                    // keep full number of CPUs instead
         maxInternalThreads = maxCPU - maxExternalThreads + 1;
     }
     else {
