@@ -113,12 +113,15 @@ private:
 
     double rad = 3.1415926535 / 180.;
 
+    float minCoaddSaturationValue = 1.e12;
+
     QString statusAtDistribute = "";
 
     instrumentDataType altInstData;   // For instruments with e.g. simultaneous optical and NIR detectors requiring different masks during HDU reformatting
 
     QMap<QString,QStringList> headerDictionary;
     QMap<QString,QString> filterDictionary;
+    QMap<QString,QString> instrumentDictionary;
 
     int recurseCounter = 0;
 
@@ -287,6 +290,7 @@ private:
     long makeAnetHeaderList(Data *scienceData);
     bool setupBackgroundList(int chip, Data *skyData, const QString &chipName);
     void combineAllBackgroundUsabilityFlags(const QList<MyImage *> &backgroundList);
+    void getMinimumSaturationValue();
 private slots:
 //    void displayRAMload();
 //    void displayMemoryTotalUsed();
@@ -312,7 +316,7 @@ private slots:
     void taskInternalGetCatalogFromWEB();
     void taskInternalGetCatalogFromIMAGE();
     void taskInternalHDUreformat();
-    void taskInternalResolveTarget();
+    void taskInternalResolveTargetSidereal();
     void taskInternalRestoreHeader();
     //    void taskInternalCopyZeroOrder();
     void taskInternalAstromphotom();
@@ -328,7 +332,8 @@ private slots:
     void registerScampSolutionAcceptance(bool scampSolutionAccepted);
     void continueWithCopyZeroOrder();
     void copyZeroOrder();
-    void errorFoundReceived();
+    void swarpErrorFoundReceived();
+    void scampErrorFoundReceived();
     void addToProgressBarReceived(const float differential);
 //    void splitterMemoryReceived(long memoryUsed);
 
@@ -471,6 +476,7 @@ public:
     QLineEdit *getDataTreeLineEdit(Data *data);
     void getFieldCenter(Data *data, QString &alphaCenter, QString &deltaCenter);
 
+    void populateInstrumentDictionary();
 signals:
     void loadViewer(QString dirname, QString filter, QString mode);
     void messageAvailable(QString message, QString type);
