@@ -73,61 +73,74 @@ void Controller::taskInternalAstromphotom()
             runAnet(scienceData);
         }
 
-        // Prepare scamp directories and perform consistency checks
-        prepareScampRun(scienceData);
-        // Collect scamp input catalogs
-        long totNumObjects = 0;
-        long numCats = prepareScampCats(scienceData, totNumObjects);
-        if (numCats == 0) return;
+        // run scamp and retrieval of point source catalog in parallel
+#pragma omp parallel sections
+        {
+#pragma omp section
+            {
+                // Prepare scamp directories and perform consistency checks
+                prepareScampRun(scienceData);
+                // Collect scamp input catalogs
+                long totNumObjects = 0;
+                long numCats = prepareScampCats(scienceData, totNumObjects);
+                //              if (numCats == 0) return;      // no exiting in openMP construct
+                if (numCats > 0) {
 
-        // Release memory:
-        // 140 bytes per detection
-        // "a few 10 kB" per FITS table
-        //
-        long totBytes = 0;
-        int degrees = cdw->ui->ASTdistortLineEdit->text().toInt();
-        totBytes += totNumObjects * 140 + (2*50000*instData->numChips)*numCats;
-        long Nt = numCats * 1 * (degrees*degrees+2) + numCats*6;
-        totBytes += 8*Nt*Nt;
-        totBytes = 2*totBytes/ 1024. / 1024.;
-        if (verbosity >= 1) emit messageAvailable("Scamp memory estimate: " + QString::number(long(totBytes)) + " MB", "controller");
-        releaseMemory(totBytes, 1);
+                    // Release memory:
+                    // 140 bytes per detection
+                    // "a few 10 kB" per FITS table
+                    //
+                    long totBytes = 0;
+                    int degrees = cdw->ui->ASTdistortLineEdit->text().toInt();
+                    totBytes += totNumObjects * 140 + (2*50000*instData->numChips)*numCats;
+                    long Nt = numCats * 1 * (degrees*degrees+2) + numCats*6;
+                    totBytes += 8*Nt*Nt;
+                    totBytes = 2*totBytes/ 1024. / 1024.;
+                    if (verbosity >= 1) emit messageAvailable("Scamp memory estimate: " + QString::number(long(totBytes)) + " MB", "controller");
+                    releaseMemory(totBytes, 1);
 
-        progress = 0.;
-        progressStepSize = 80. / (float) numCats;  // 80%, leaving some space for distortion solution.
-        // Build the scamp command
-        buildScampCommand(scienceData);
+                    progress = 0.;
+                    progressStepSize = 80. / (float) numCats;  // 80%, leaving some space for distortion solution.
+                    // Build the scamp command
+                    buildScampCommand(scienceData);
 
-        // Run the Scamp command
-        workerThread = new QThread();
-        scampWorker = new ScampWorker(scampCommand, scampDir, instData->shortName);
-        workerInit = true;
-        workerThreadInit = true;
-        scampWorker->moveToThread(workerThread);
+                    // Run the Scamp command
+                    workerThread = new QThread();
+                    scampWorker = new ScampWorker(scampCommand, scampDir, instData->shortName);
+                    workerInit = true;
+                    workerThreadInit = true;
+                    scampWorker->moveToThread(workerThread);
 
-        connect(workerThread, &QThread::started, scampWorker, &ScampWorker::runScamp);
-        // Qt::DirectConnection is bad here, because this task runs in a different thread than the main controller,
-        // meaning that if e.g. sky sub is activated as well it will start immediately even before the scamp checkplots are shown
-        // connect(workerThread, &QThread::finished, workerThread, &QThread::deleteLater, Qt::DirectConnection);
-        // connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit, Qt::DirectConnection);
-        // connect(scampWorker, &ScampWorker::finished, scampWorker, &QObject::deleteLater, Qt::DirectConnection);
-        connect(workerThread, &QThread::finished, workerThread, &QThread::deleteLater);
-        connect(scampWorker, &ScampWorker::errorFound, this, &Controller::errorFoundReceived);
-        connect(scampWorker, &ScampWorker::finishedScamp, this, &Controller::finishedScampReceived);
-        // Need the direct connection if we want the thread to actually return control to the main thread (activating the start button again).
-        // But then sky sub would start before checkplots are evaluated.
-        // CORRECT WAY: call workerThread->quit() at the end of the image quality analysis
-        //        connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit, Qt::DirectConnection);
-        //       connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit);
-        connect(scampWorker, &ScampWorker::finished, scampWorker, &QObject::deleteLater);
-        connect(scampWorker, &ScampWorker::messageAvailable, monitor, &Monitor::displayMessage);
-        connect(scampWorker, &ScampWorker::fieldMatched, this, &Controller::fieldMatchedReceived);
-        workerThread->start();
-        workerThread->wait();
+                    connect(workerThread, &QThread::started, scampWorker, &ScampWorker::runScamp);
+                    // Qt::DirectConnection is bad here, because this task runs in a different thread than the main controller,
+                    // meaning that if e.g. sky sub is activated as well it will start immediately even before the scamp checkplots are shown
+                    // connect(workerThread, &QThread::finished, workerThread, &QThread::deleteLater, Qt::DirectConnection);
+                    // connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit, Qt::DirectConnection);
+                    // connect(scampWorker, &ScampWorker::finished, scampWorker, &QObject::deleteLater, Qt::DirectConnection);
+                    connect(workerThread, &QThread::finished, workerThread, &QThread::deleteLater);
+                    connect(scampWorker, &ScampWorker::errorFound, this, &Controller::scampErrorFoundReceived);
+                    connect(scampWorker, &ScampWorker::finishedScamp, this, &Controller::finishedScampReceived);
+                    // Need the direct connection if we want the thread to actually return control to the main thread (activating the start button again).
+                    // But then sky sub would start before checkplots are evaluated.
+                    // CORRECT WAY: call workerThread->quit() at the end of the image quality analysis
+                    //       connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit, Qt::DirectConnection);
+                    //       connect(scampWorker, &ScampWorker::finished, workerThread, &QThread::quit);
+                    connect(scampWorker, &ScampWorker::finished, scampWorker, &QObject::deleteLater);
+                    connect(scampWorker, &ScampWorker::messageAvailable, monitor, &Monitor::displayMessage);
+                    connect(scampWorker, &ScampWorker::fieldMatched, this, &Controller::fieldMatchedReceived);
+                    workerThread->start();
+                    workerThread->wait();
+                }
+            }
+#pragma omp section
+            {
+                downloadGaiaCatalog(scampScienceData); // Point sources
+            }
+        }
     }
 
-    // Currently not active, and does not write final headers/*.head files
-    if (cdw->ui->ASTmethodComboBox->currentText() == "astrometry.net") {
+    // Not implemented (greyed out)
+    if (cdw->ui->ASTmethodComboBox->currentText() == "Astrometry.net") {
         QString catDirName = mainDirName + "/" + scienceDir + "/cat/";
         if (!scienceData->hasMatchingPartnerFiles(catDirName, ".anet")) return;
 
@@ -164,6 +177,13 @@ void Controller::taskInternalAstromphotom()
     }
 }
 
+void Controller::scampErrorFoundReceived()
+{
+    successProcessing = false;
+    criticalReceived();
+    workerThread->quit();
+}
+
 int Controller::getMaxPhotInst()
 {
     if (!successProcessing) return 0;
@@ -203,6 +223,7 @@ void Controller::scampCalcFluxscale()
         successProcessing = false;
         emit messageAvailable("Could not find scamp output headers.", "error");
         criticalReceived();
+        workerThread->quit();
         return;
     }
 
@@ -236,7 +257,11 @@ void Controller::scampCalcFluxscale()
         MEF.close();
 
         // Find the associated exposure time
-        for (auto &it : scampScienceData->myImageList[0]) {
+        if (instData->validChip == -1) {
+            successProcessing = false;
+            return;
+        }
+        for (auto &it : scampScienceData->myImageList[instData->validChip]) {
             it->loadHeader();
             if (it->baseName.contains(MEFbasename)) {
                 exptime = it->exptime;
@@ -300,8 +325,8 @@ void Controller::splitScampHeaders()
     QStringList MEFheaders = dir.entryList(filterList);
     bool success = true;
     long MEFindex = 0;
+
     for (auto &it : MEFheaders) {
-        int chip = 1;
         QFile MEF(scampDir+"/"+it);
         QFileInfo MEFinfo(MEF);
         QTextStream inStream(&MEF);
@@ -313,18 +338,19 @@ void Controller::splitScampHeaders()
         QTextStream outStream;
         QString line;
         int i=0;
+        int headCount = 0;
         while (inStream.readLineInto(&line)) {
             if (i==0) {
-                if (instData->badChips.contains(chip-1)) ++chip;
+                int chip = instData->chipMap.key(headCount) + 1;      // dealing with user defined bad chips
                 HEAD.setFileName(scampHeadersDir+"/"+MEFinfo.completeBaseName()+"_"+QString::number(chip)+".head");
                 outStream.setDevice(&HEAD);
-                if( !HEAD.open(QIODevice::WriteOnly)) {
+                if (!HEAD.open(QIODevice::WriteOnly)) {
                     success = false;
                     break;
                 }
             }
             // THELI style flux scaling
-            if (line.contains("FLXSCALE=")) line.replace("FLXSCALE=","FLSCALE =");        // preserve the original FLXSCALE value
+            if (line.contains("FLXSCALE=")) line.replace("FLXSCALE=","FLSCALE =");        // copying the original FLXSCALE keyword to FLSCALE (original will be replaced below)
             if (line.contains("PHOTINST")) {
                 int photinst_idx = line.split("=")[1].split("/")[0].simplified().toInt() - 1;
                 long idx = photInstruments[photinst_idx]->indexMap.value(MEFindex);
@@ -339,7 +365,7 @@ void Controller::splitScampHeaders()
                 HEAD.close();
                 HEAD.setPermissions(QFile::ReadUser | QFile::WriteUser);
                 i=0;
-                ++chip;
+                ++headCount;
             }
         }
         MEF.close();
@@ -438,9 +464,9 @@ void Controller::copyZeroOrder()
                 continue;
             }
             //            it->backupOrigHeader(chip);            // Create a backup copy of the original FITS headers if it doesn't exist yet
-            if (it->scanAstromHeader(chip, "inHeadersDir")) {    // reads the header, and updates the wcs struct in MyImage class
+            if (it->scanAstromHeader(chip, "inHeadersDir")) {    // reads the header, and updates the wcs struct in MyImage class (i.e. the memory)
                 it->updateZeroOrderOnDrive("update");            // Overwrite 0-th order solution in FITS header (if on drive)
- //               it->updateZeroOrderInMemory();                   // Overwrite 0-th order solution in memory
+                //               it->updateZeroOrderInMemory();                   // Overwrite 0-th order solution in memory
             }
             // The following is done by scanAstromheader and updateZeroOrderOnDrive
             /*
@@ -601,7 +627,7 @@ void Controller::doImageQualityAnalysis()
     pushBeginMessage("ImageQuality", scampScienceDir);
 
     // Todo: replace with GAIA catalog if that was the reference catalog already!
-    downloadGaiaCatalog(scampScienceData); // Point sources
+    //    downloadGaiaCatalog(scampScienceData); // Point sources
 
     QList<MyImage*> allMyImages;
     long numMyImages = makeListofAllImages(allMyImages, scampScienceData);
@@ -631,14 +657,14 @@ void Controller::doImageQualityAnalysis()
         it->collectSeeingParameters(imageQuality->sourceCat, imageQuality->sourceMag, instData->chipMap.value(chip));
         // match
         static_cast<void> (imageQuality->getSeeingFromGaia());
-        it->fwhm = imageQuality->fwhm;                                                     // Updating MyImage fwhm parameter
-        it->updateHeaderValue("FWHM", imageQuality->fwhm);                                 // Updating MyImage header string
+        it->fwhm = imageQuality->fwhm * instData->pixscale;                                // Updating MyImage fwhm parameter, in arcsec
+        it->updateHeaderValue("FWHM", it->fwhm);                                           // Updating MyImage header string
         it->updateHeaderValue("ELLIP", imageQuality->ellipticity);
-        it->updateHeaderValueInFITS("FWHM", QString::number(imageQuality->fwhm, 'f', 3));  // Updating the current FITS image on drive
+        it->updateHeaderValueInFITS("FWHM", QString::number(it->fwhm, 'f', 3));            // Updating the current FITS image on drive
         it->updateHeaderValueInFITS("ELLIP", QString::number(imageQuality->ellipticity, 'f', 3));
         //        if (!gaia) imageQuality->getSeeingFromRhMag();      TODO: Not yet implemented
         if (verbosity > 1) emit messageAvailable(it->chipName + " : FWHM / Ellipticity / # stars = "
-                                                 + QString::number(imageQuality->fwhm, 'f', 3) + " / "
+                                                 + QString::number(it->fwhm, 'f', 3) + " / "
                                                  + QString::number(imageQuality->ellipticity, 'f', 3) + " / "
                                                  + QString::number(imageQuality->numSources), "ignore");
         delete imageQuality;
@@ -876,7 +902,9 @@ long Controller::getNumObjectsScampCat(QString cat)
     }
     fits_close_file(fptr, &status);
 
-    printCfitsioError("getNumObjectsScampCat():<br>" + cat, status);
+    QString out = __func__;
+    out.append( " :<br> " +cat);
+    printCfitsioError(out, status);
 
     return nobj;
 }
@@ -923,7 +951,7 @@ long Controller::getNumObjectsSourceExtractorCat(QString cat)
     fits_get_num_rows(fptr, &nrows, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("getNumObjectsSourceExtractorCat():", status);
+    printCfitsioError(__func__, status);
 
     return nrows;
 }
