@@ -52,7 +52,7 @@ void Controller::taskInternalProcessbias()
 
     // TODO: The following line is needed only as long as we are handling splitting of raw data by scripts.
     // The biasData imagelist is empty after splitting because the Data constructor only looks for *_chip.fits, not for raw files.
-    if (biasData->myImageList[0].isEmpty()) {
+    if (biasData->myImageList[instData->validChip].isEmpty()) {
         biasData->populate("");
         emit populateMemoryView();
     }
@@ -60,7 +60,7 @@ void Controller::taskInternalProcessbias()
     getNumberOfActiveImages(biasData);
 
     // Release as much memory as maximally necessary
-    float nimg = biasData->myImageList[0].length() + 1;  // The number of images one thread keeps in memory
+    float nimg = biasData->myImageList[instData->validChip].length() + 1;  // The number of images one thread keeps in memory
     releaseMemory(nimg*instData->storage*maxExternalThreads, 1, "calibrator");
     // Protect the rest, will be unprotected as needed
     biasData->protectMemory();
@@ -93,7 +93,13 @@ void Controller::taskInternalProcessbias()
             progress += progressHalfStepSize;
         }
 
-        // Commented to avoid crash depending on tests
+        if (biasData->myImageList[chip].length() == 0) {
+            biasData->successProcessing = false;
+            successProcessing = false;
+            emit messageAvailable("Could not find data for chip "+QString::number(chip+1)+ " in "+biasData->subDirName, "error");
+            emit criticalReceived();
+            continue;
+        }
         biasData->combineImagesCalib(chip, combineBias_ptr, nlow, nhigh, dataDirName, dataSubDirName, dataDataType);  // Combine images
         biasData->getModeCombineImages(chip);
         biasData->writeCombinedImage(chip);
@@ -108,6 +114,10 @@ void Controller::taskInternalProcessbias()
 #pragma omp atomic
         progress += progressCombinedStepSize;
     }
+    if (!successProcessing) {
+        emit progressUpdate(100);
+        return;
+    }
     biasData->reportModeCombineImages();
 
     checkSuccessProcessing(biasData);
@@ -142,13 +152,13 @@ void Controller::taskInternalProcessdark()
     getNumberOfActiveImages(darkData);
 
     // TODO: The following line is needed only as long as we are handling splitting of raw data by scripts.
-    if (darkData->myImageList[0].isEmpty()) {
+    if (darkData->myImageList[instData->validChip].isEmpty()) {
         darkData->populate("");
         emit populateMemoryView();
     }
 
     // Release as much memory as maximally necessary
-    float nimg = darkData->myImageList[0].length() + 1;  // The number of images one thread keeps in memory
+    float nimg = darkData->myImageList[instData->validChip].length() + 1;  // The number of images one thread keeps in memory
     releaseMemory(nimg*instData->storage*maxExternalThreads, 1, "calibrator");
     // Protect the rest, will be unprotected as needed
     darkData->protectMemory();
@@ -175,6 +185,13 @@ void Controller::taskInternalProcessdark()
 #pragma omp atomic
             progress += progressHalfStepSize;
         }
+        if (darkData->myImageList[chip].length() == 0) {
+            darkData->successProcessing = false;
+            successProcessing = false;
+            emit messageAvailable("Could not find data for chip "+QString::number(chip+1)+ " in "+darkData->subDirName, "error");
+            emit criticalReceived();
+            continue;
+        }
         darkData->combineImagesCalib(chip, combineDark_ptr, nlow, nhigh, dataDirName, dataSubDirName, dataDataType);
         darkData->getModeCombineImages(chip);
         darkData->writeCombinedImage(chip);
@@ -186,6 +203,11 @@ void Controller::taskInternalProcessdark()
 #pragma omp atomic
         progress += progressCombinedStepSize;
     }
+    if (!successProcessing) {
+        emit progressUpdate(100);
+        return;
+    }
+
     darkData->reportModeCombineImages();
 
     checkSuccessProcessing(darkData);
@@ -216,7 +238,7 @@ void Controller::taskInternalProcessflatoff()
     pushConfigProcessflatoff();
 
     // TODO: The following line is needed only as long as we are handling splitting of raw data by scripts.
-    if (flatoffData->myImageList[0].isEmpty()) {
+    if (flatoffData->myImageList[instData->validChip].isEmpty()) {
         flatoffData->populate("");
         emit populateMemoryView();
     }
@@ -224,7 +246,7 @@ void Controller::taskInternalProcessflatoff()
     getNumberOfActiveImages(flatoffData);
 
     // Release as much memory as maximally necessary
-    float nimg = flatoffData->myImageList[0].length() + 1;  // The number of images one thread keeps in memory
+    float nimg = flatoffData->myImageList[instData->validChip].length() + 1;  // The number of images one thread keeps in memory
     releaseMemory(nimg*instData->storage*maxExternalThreads, 1, "calibrator");
     // Protect the rest, will be unprotected as needed
     flatoffData->protectMemory();
@@ -251,6 +273,13 @@ void Controller::taskInternalProcessflatoff()
 #pragma omp atomic
             progress += progressHalfStepSize;
         }
+        if (flatoffData->myImageList[chip].length() == 0) {
+            flatoffData->successProcessing = false;
+            successProcessing = false;
+            emit messageAvailable("Could not find data for chip "+QString::number(chip+1)+ " in "+flatoffData->subDirName, "error");
+            emit criticalReceived();
+            continue;
+        }
         flatoffData->combineImagesCalib(chip, combineFlatoff_ptr, nlow, nhigh, dataDirName, dataSubDirName, dataDataType);
         flatoffData->getModeCombineImages(chip);
         flatoffData->writeCombinedImage(chip);
@@ -262,6 +291,11 @@ void Controller::taskInternalProcessflatoff()
 #pragma omp atomic
         progress += progressCombinedStepSize;
     }
+    if (!successProcessing) {
+        emit progressUpdate(100);
+        return;
+    }
+
     flatoffData->reportModeCombineImages();
 
     checkSuccessProcessing(flatoffData);
@@ -336,7 +370,7 @@ void Controller::taskInternalProcessflat()
     memoryDecideDeletableStatus(flatData, false);
 
     // TODO: The following line is needed only as long as we are handling splitting of raw data by scripts.
-    if (flatData->myImageList[0].isEmpty()) {
+    if (flatData->myImageList[instData->validChip].isEmpty()) {
         flatData->populate("");
         emit populateMemoryView();
     }
@@ -344,7 +378,7 @@ void Controller::taskInternalProcessflat()
     getNumberOfActiveImages(flatData);
 
     // Release as much memory as maximally necessary
-    float nimg = flatData->myImageList[0].length() + 2;  // The number of images one thread keeps in memory
+    float nimg = flatData->myImageList[instData->validChip].length() + 2;  // The number of images one thread keeps in memory
     releaseMemory(nimg*instData->storage*maxExternalThreads, 1, "calibrator");
     // Protect the rest, will be unprotected as needed
     flatData->protectMemory();
@@ -381,19 +415,26 @@ void Controller::taskInternalProcessflat()
             it->checkCorrectMaskSize(instData);
             it->setModeFlag(min, max);                       // Must set mode flags before subtracting dark component (flatoffs can have really high levels in NIR data)
             if (biasData != nullptr && biasData->successProcessing) { // cannot pass nullptr to subtractBias()
-//                it->subtractBias(ref, biasDataType);
+                //                it->subtractBias(ref, biasDataType);
                 it->subtractBias(biasData->combinedImage[chip], biasDataType);
-//                  it->subtractBias(biasData->combinedImage[chip]->dataCurrent, biasDataType);
+                //                  it->subtractBias(biasData->combinedImage[chip]->dataCurrent, biasDataType);
                 it->skyValue -= biasData->combinedImage[chip]->skyValue;
                 it->saturationValue -= biasData->combinedImage[chip]->skyValue;
             }
-//            it->getMode(true);
-//            it->setModeFlag(min, max);
+            //            it->getMode(true);
+            //            it->setModeFlag(min, max);
             if (!it->successProcessing) flatData->successProcessing = false;  // does not need to be threadsafe
 # pragma omp atomic
             progress += progressHalfStepSize;
         }
         if (biasData != nullptr) biasData->unprotectMemory(chip);
+        if (flatData->myImageList[chip].length() == 0) {
+            flatData->successProcessing = false;
+            successProcessing = false;
+            emit messageAvailable("Could not find data for chip "+QString::number(chip+1)+ " in "+flatData->subDirName, "error");
+            emit criticalReceived();
+            continue;
+        }
         flatData->combineImagesCalib(chip, combineFlat_ptr, nlow, nhigh, dataDirName, dataSubDirName, dataDataType);
         // Remove Bayer intensity variations within a 2x2 superpixel
         if (!instData->bayer.isEmpty()) equalizeBayerFlat(flatData->combinedImage[chip]);
@@ -415,6 +456,11 @@ void Controller::taskInternalProcessflat()
         // flatData->combinedImage[chip]->illuminationCorrection(chip, thelidir, instData->name, filter);
     }
 
+    if (!successProcessing) {
+        emit progressUpdate(100);
+        return;
+    }
+
     // Obtain gain normalizations. Must be done once all images have been combined.
     flatData->getGainNormalization();
 
@@ -495,7 +541,7 @@ void Controller::taskInternalProcessscience()
 
     memoryDecideDeletableStatus(scienceData, false);
     // TODO: The following line is needed only as long as we are handling splitting of raw data by scripts.
-    if (scienceData->myImageList[0].isEmpty()) {
+    if (scienceData->myImageList[instData->validChip].isEmpty()) {
         scienceData->populate("");
         emit populateMemoryView();
     }
@@ -548,6 +594,21 @@ void Controller::taskInternalProcessscience()
         doDataFitInRAM(4*numMyImages*instData->numUsedChips, instData->storage);
     }
 
+    if (instData->bayer.isEmpty()) {
+        QStringList filters;
+        for (auto &it : allMyImages) {
+            filters.append(it->filter);
+        }
+        filters.removeDuplicates();
+        if (filters.length() > 1) {
+            QString filtersJoined = filters.join(", ");
+            emit messageAvailable("Files with different FILTER keywords found in "+scienceData->dirName+" :", "error");
+            emit messageAvailable(filtersJoined, "error");
+            emit warningReceived();
+            return;
+        }
+    }
+
     // Must keep track of memory consumption
     scienceData->bayerList.clear();
     scienceData->bayerList.resize(instData->numChips);
@@ -567,18 +628,18 @@ void Controller::taskInternalProcessscience()
         // Don't remember why we need a lock here. I think it had to do with the headers. Will crash otherwise
         // TODO: probably not needed anymore with latest memory scheme
         QString message;
-// #pragma omp critical
-//        {
-            if (biasData != nullptr) {
-                biasData->loadCombinedImage(chip);  // skipped if already in memory
-                message.append(biasData->subDirName);
-            }
-            if (flatData != nullptr) {
-                flatData->loadCombinedImage(chip);  // skipped if already in memory
-                if (message.isEmpty()) message.append(flatData->subDirName);
-                else message.append(" and " + flatData->subDirName);
-            }
-//        }
+        // #pragma omp critical
+        //        {
+        if (biasData != nullptr) {
+            biasData->loadCombinedImage(chip);  // skipped if already in memory
+            message.append(biasData->subDirName);
+        }
+        if (flatData != nullptr) {
+            flatData->loadCombinedImage(chip);  // skipped if already in memory
+            if (message.isEmpty()) message.append(flatData->subDirName);
+            else message.append(" and " + flatData->subDirName);
+        }
+        //        }
         if (verbosity >= 0) {
             if (!message.isEmpty() && instData->bayer.isEmpty()) emit messageAvailable(it->chipName + " : Correcting with "+message, "image");
             if (!message.isEmpty() && !instData->bayer.isEmpty()) emit messageAvailable(it->chipName + " : Correcting with "+message+", debayering", "image");
