@@ -160,7 +160,7 @@ void Controller::detectionInternal(Data *scienceData, QString minFWHM, QString m
         it->checkWCSsanity();
         it->readWeight();
         it->backgroundModel(256, "interpolate");
-        it->updateSaturation(saturation);
+//        it->updateSaturation(saturation);        // Handled during splitting
         if (it->dataBackground.capacity() == 0) {
             if (successProcessing) emit messageAvailable(it->chipName + " : Background vector has zero capacity!", "error");
             criticalReceived();
@@ -389,7 +389,7 @@ void Controller::mergeInternal(Data *scienceData, QString minFWHM, QString maxFl
 
         fits_close_file(fptr, &status);
 
-        printCfitsioError("mergeInternal()", status);
+        printCfitsioError(__func__, status);
 
 #pragma omp atomic
         progress += progressStepSize;
@@ -427,7 +427,7 @@ void Controller::mergeSourceExtractor(Data *scienceData)
         }
         fits_close_file(fptr, &status);
 
-        printCfitsioError("mergeSourceExtractor()", status);
+        printCfitsioError(__func__, status);
 
 #pragma omp atomic
         progress += progressStepSize;
