@@ -451,7 +451,7 @@ QString Query::filterStringToVizierName(QString filter)
     return "";
 }
 
-QString Query::resolveTarget(QString target)
+QString Query::resolveTargetSidereal(QString target)
 {
     if (!successProcessing) return "Unresolved";
 
@@ -466,6 +466,42 @@ QString Query::resolveTarget(QString target)
         line = line.simplified();
         if (line.contains("Traceback")) return "Unresolved";
         else {
+            // DEPRECATED. GUI will not launch if python3 cannot be found. The following is kept for reference only.
+            // In case python2 is used instead of python3, we have to remove parentheses and commas from the output
+            line = line.remove("(");
+            line = line.remove(")");
+            line = line.replace(",", " ");
+            line = line.simplified();
+            QStringList coordList = line.split(' ');
+            targetAlpha = coordList[0];
+            targetDelta = coordList[1];
+            // Convert to HHMMSS and DDMMSS
+            if (!targetAlpha.contains(":")) targetAlpha = decimalToHms(targetAlpha.toDouble());
+            if (!targetDelta.contains(":")) targetDelta = decimalToDms(targetDelta.toDouble());
+            return "Resolved";
+        }
+    }
+    return "Unresolved";
+}
+
+QString Query::resolveTargetNonsidereal(QString target, QString dateobs, float geoLon, float geoLat)
+{
+    if (!successProcessing) return "Unresolved";
+
+    QString geolonString = QString::number(geoLon, 'f', 4)+"d";
+    QString geolatString = QString::number(geoLat, 'f', 4)+"d";
+    queryCommand = pythonExecutable + " " + thelidir+"/python/mpc_query.py "+target + " " + dateobs + " " + geolonString + " " + geolatString;
+
+    runCommand(queryCommand);
+
+    QTextStream stream(&byteArray, QIODevice::ReadOnly);
+
+    QString line;
+    while (stream.readLineInto(&line)) {
+        line = line.simplified();
+        if (line.contains("Traceback")) return "Unresolved";
+        else {
+            // DEPRECATED. GUI will not launch if python3 cannot be found. The following is kept for reference only.
             // In case python2 is used instead of python3, we have to remove parentheses and commas from the output
             line = line.remove("(");
             line = line.remove(")");
@@ -822,6 +858,7 @@ void Query::pushNumberOfSources()
             messageString = "WARNING: No reference sources retrieved! <br>"
                             "-- try a different catalog <br>"
                             "-- check your internet connection <br>"
+                            "-- do not run THELI from a console with active conda/astroconda environment <br>"
                             "-- adjust mag limit and radius";
             emit messageAvailable(messageString, "stop");
             emit critical();
@@ -1265,7 +1302,7 @@ void Query::writeAstromScamp()
     //    fits_write_col(fptr, TSHORT, 8, firstrow, firstelem, nrows, fieldpos, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("writeAstromScamp()", status);
+    printCfitsioError(__func__, status);
 
     QFile file(filename);
     file.setPermissions(QFile::ReadUser | QFile::WriteUser);
@@ -1305,8 +1342,8 @@ void Query::writeAstromANET()
     int tfields = 3;
     long nrows = numSources;
 
-    if (numSources > 50000) {
-        QString messageString = "NOTE: More than 50000 sources were retrieved.\n"
+    if (numSources > 150000) {
+        QString messageString = "NOTE: More than 1cd ../TEST350000 sources were retrieved.\n"
                                 "The astrometry.net index is not being built as it is very time consuming.\n"
                                 "If you want to use astrometry.net, reduce the catalog size by imposing\n"
                                 "a (tighter) magnitude limit.";
@@ -1326,7 +1363,7 @@ void Query::writeAstromANET()
     fits_write_col(fptr, TFLOAT, 3, firstrow, firstelem, nrows, mag_arr, &status);
     fits_close_file(fptr, &status);
 
-    printCfitsioError("writeAstromANET()", status);
+    printCfitsioError(__func__, status);
 
     // STEP 2: build the anet index
 
