@@ -172,6 +172,7 @@ QStringList MainWindow::createCommandlistBlock(QString taskBasename, QStringList
     }
 
     DataDir datadir;
+    datadir.numChips = instData.numChips;
     QStringList commandList;
 
     // Process science can act on DT_SCIENCE, SKY, STD and thus needs to distinguish between them
@@ -184,7 +185,7 @@ QStringList MainWindow::createCommandlistBlock(QString taskBasename, QStringList
 
     // Loop over all directories found for the current task
     for (auto &it : goodDirList) {
-        if (taskBasename != "ResolveTarget") datadir.setPaths(it);   // triggers  qdebug() message if just resolving a target
+        if (taskBasename != "ResolveTargetSidereal") datadir.setPaths(it);   // triggers  qdebug() message if just resolving a target
         // Now call a consistency check by the function's string representation.
         // The function name is "check_task<taskBasename>()"
         // and it simply updates the 'stop' and 'skip' flags
@@ -224,7 +225,7 @@ QStringList MainWindow::createCommandlistBlock(QString taskBasename, QStringList
                 || taskBasename == "Globalweight"
                 || taskBasename == "Individualweight"
                 || taskBasename == "Skysub"
-                || taskBasename == "ResolveTarget"
+                || taskBasename == "ResolveTargetSidereal"
                 || taskBasename == "GetCatalogFromIMAGE"
                 || taskBasename == "RestoreHeader"
                 || taskBasename == "Separate") {
@@ -539,9 +540,9 @@ void MainWindow::on_startPushButton_clicked()
     }
 
     // Check if all data Dirs are valid. If there is a single one that isn't, abort!
-    // Exception: mode = ResolveTarget does not require any data directories
+    // Exception: mode = ResolveTargetSidereal does not require any data directories
     if (! areAllPathsValid()
-            && (mode != "ResolveTarget")
+            && (mode != "ResolveTargetSidereal")
             && !doingInitialLaunch) {
         message(ui->plainTextEdit, "STOP: Nonexistent entries were found in the data directory tree. "
                                    "They must be fixed or removed.");
@@ -560,7 +561,7 @@ void MainWindow::on_startPushButton_clicked()
     // Process commands from other buttons than the "start" button
     if (mode == "GetCatalogFromWEB"
             || mode == "GetCatalogFromIMAGE"
-            || mode == "ResolveTarget"
+            || mode == "ResolveTargetSidereal"
             || mode == "RestoreHeader") {
         taskBasename = mode;
         if (!OSPBC_addCommandBlock(taskBasename, mode, stop)) return;   // updates totalCommandList
@@ -589,7 +590,7 @@ void MainWindow::on_startPushButton_clicked()
         // Leave if no normal task is checked (apart from special buttons)
         if (mode != "GetCatalogFromWEB"
                 && mode != "GetCatalogFromIMAGE"
-                && mode != "ResolveTarget"
+                && mode != "ResolveTargetSidereal"
                 && mode != "RestoreHeader"
                 && !anythingChecked) {
             return;
@@ -629,7 +630,7 @@ void MainWindow::on_startPushButton_clicked()
         workerThread->start();
 
         // show the process monitor, for normal processing tasks, only
-        if (taskBasename != "ResolveTarget") {
+        if (taskBasename != "ResolveTargetSidereal") {
             if (switchProcessMonitorPreference) monitor->raise();
             ui->processProgressBar->setValue(0);
         }
@@ -750,7 +751,7 @@ QString MainWindow::OSPBC_determineExecutionMode(QObject *sender)
         if (cdw->ui->ARCimageRadioButton->isChecked()) mode = "GetCatalogFromIMAGE";
     }
     else if (sender == cdw->ui->restoreHeaderPushButton) mode = "RestoreHeader";
-    else if (sender == cdw->ui->ARCtargetresolverToolButton) mode = "ResolveTarget";
+    else if (sender == cdw->ui->ARCtargetresolverToolButton) mode = "ResolveTargetSidereal";
     else {
         mode = "simulate";
         ui->plainTextEdit->clear();
@@ -769,7 +770,7 @@ bool MainWindow::OSPBC_multipleDirConsistencyCheck()
     for (auto &it : status.listCheckBox) {
         if (it->isChecked()) {
             QString taskBasename = it->objectName().remove("apply").remove("CheckBox");
-            test *= checkMultipledirConsistency(taskBasename);
+            test = test && checkMultipledirConsistency(taskBasename);
         }
         ++i;
         // Don't have to evaluate more checkboxes.
@@ -1000,7 +1001,19 @@ QStringList MainWindow::displayCoaddFilterChoice(QString dirname, QString &filte
 
     // We look in normal images, then in skysub images for filter string
     QDir dir(dirname);
-    QStringList filter("*_1*.fits");
+    // Figure out a chip that must be present (not excluded by the user
+    int testChip = -1;
+    for (int chip=0; chip<instData.numChips; ++chip) {
+        if (!instData.badChips.contains(chip)) {
+            testChip = chip;
+            break;
+        }
+    }
+    if (testChip == -1) {
+        qDebug() << __func__ << "error: no data left after filtering";
+    }
+
+    QStringList filter("*_"+QString::number(testChip+1)+"*.fits");
     dir.setNameFilters(filter);
     dir.setSorting(QDir::Name);
     QStringList list = dir.entryList();
@@ -1017,7 +1030,7 @@ QStringList MainWindow::displayCoaddFilterChoice(QString dirname, QString &filte
         fits_open_file(&fptr, (dirname+"/"+fits).toUtf8().data(), READONLY, &status);
         fits_read_key_str(fptr, "FILTER", filter, NULL, &status);
         fits_close_file(fptr, &status);
-        printCfitsioError(fits+" : MainWindow::displayCoaddFilterChoice()", status);
+        printCfitsioError(fits+" : " + __func__, status);
         QString filterString(filter);
         filterString = filterString.simplified();
         if (!filterList.contains(filterString)) filterList.append(filterString);
@@ -1066,7 +1079,6 @@ QStringList MainWindow::displayCoaddFilterChoice(QString dirname, QString &filte
 void MainWindow::printCfitsioError(QString funcName, int status)
 {
     if (status) {
-        qDebug() << status;
         CfitsioErrorCodes *errorCodes = new CfitsioErrorCodes(this);
         emit messageAvailable(funcName+":<br>" + errorCodes->errorKeyMap.value(status), "error");
     }
