@@ -72,6 +72,7 @@ Splitter::Splitter(const instrumentDataType &instrumentData, const Mask *detecto
 
 void Splitter::backupRawFile()
 {
+    if (!successProcessing) return;
     moveFile(fileName, path, path+"/RAWDATA/");
 }
 
@@ -133,13 +134,14 @@ void Splitter::determineFileFormat()
         QDir unknownFile(path+"/UnknownFormat");
         unknownFile.mkpath(path+"/UnknownFormat/");
         moveFile(fileName, path, path+"/UnknownFormat");
-        emit messageAvailable(fileName+" : Unknown format. Cfitsio error code: "+QString::number(rawStatus) + ". Moved to "+subDirName+"/UnknownFormat/", "warning");
+        emit messageAvailable("File "+fileName+" : Unknown format. Cfitsio error code: "+QString::number(rawStatus) + ". Moved to "+subDirName+"/UnknownFormat/", "warning");
     }
 }
 
 void Splitter::uncompress()
 {
     if (!successProcessing) return;
+
     // Unpack tile-compressed image and write new fits file to disk
     if (fits_is_compressed_image(rawFptr, &rawStatus)) {
         if (*verbosity > 1) emit messageAvailable(baseName + " : Uncompressing ...", "image");
@@ -164,19 +166,16 @@ void Splitter::uncompress()
             fits_open_file(&rawFptr, name.toUtf8().data(), READONLY, &rawStatus);
             if (rawStatus) {
                 emit messageAvailable(baseName + " : Could not open uncompressed FITS file!", "error");
-                printCfitsioError("uncompress()", rawStatus);
+                printCfitsioError(__func__, rawStatus);
             }
         }
         else {
             emit messageAvailable(fi.fileName() + " : Uncompression did not work as expected!", "error");
-            printCfitsioError("uncompress()", rawStatus);
-            printCfitsioError("uncompress()", outRawStatus);
+            printCfitsioError(__func__, rawStatus);
+            printCfitsioError(__func__, outRawStatus);
             successProcessing = false;
         }
     }
-
-    // CHECK: xtalk correction only works if we maintain the original detector geometry
-    //correctXtalk();
 }
 
 void Splitter::consistencyChecks()
@@ -200,12 +199,11 @@ void Splitter::consistencyChecks()
         // equal or more numHDUs than chips is fine (e.g. multi-channel cameras, or multiple readout ports per detector)
     }
 
-    printCfitsioError("consistencyChecks()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
 }
 
 void Splitter::compileNumericKeys()
 {
-
     numericKeyNames.clear();
     numericKeyNames.append(headerDictionary.value("CRVAL1"));
     numericKeyNames.append(headerDictionary.value("CRVAL2"));
@@ -227,7 +225,6 @@ void Splitter::compileNumericKeys()
     numericKeyNames.append(headerDictionary.value("LST"));
 }
 
-
 void Splitter::extractImages()
 {
     if (!successProcessing) return;
@@ -238,15 +235,20 @@ void Splitter::extractImages()
 
     // adjust progress step size for multi-chip cameras whose detectors are stored in single extension FITS files
     // (i.e. raw data does not come as a MEF but as separate FITS files)
-    QStringList instruments = {"FORS1_E2V_2x2@VLT", "FORS2_E2V_2x2@VLT", "FORS2_MIT_1x1@VLT", "FORS2_MIT_2x2@VLT", "FourStar@LCO",
+    QStringList instruments = {"Direct_4k_SWOPE@LCO", "FORS1_E2V_2x2@VLT", "FORS2_E2V_2x2@VLT", "FORS2_MIT_1x1@VLT", "FORS2_MIT_2x2@VLT", "FourStar@LCO",
+                               "HSC@SUBARU", "LDSS3_2004-201401@LCO", "LDSS3_from201402@LCO",
+                               "IMACS_F2_200404-2008@LCO", "IMACS_F2_2008-today@LCO", "IMACS_F4_200404-2012@LCO", "IMACS_F4_2012-today@LCO", "IMACS_F4_2012-today_2x2@LCO",
                                "MOIRCS_200406-201006@SUBARU", "MOIRCS_201007-201505@SUBARU", "MOIRCS_201512-today@SUBARU",
                                "SPARTAN@SOAR", "SuprimeCam_200101-200104@SUBARU", "SuprimeCam_200105-200807@SUBARU", "SuprimeCam_200808-201705@SUBARU",
                                "SuprimeCam_200808-201705_SDFRED@SUBARU", "VIMOS@VLT"};
 
     // multiple readout channels in different FITS extensions
-    multiChannelMultiExt << "GMOS-N-HAM@GEMINI" << "GMOS-N-HAM_1x1@GEMINI"
+    multiChannelMultiExt << "DEIMOS_2AMP@KECK"
+                         << "GMOS-N-HAM@GEMINI" << "GMOS-N-HAM_1x1@GEMINI"
                          << "GMOS-S-HAM@GEMINI" << "GMOS-S-HAM_1x1@GEMINI"
-                         << "SOI@SOAR";
+                         << "LRIS_BLUE@KECK" << "LRIS_RED@KECK"
+                         << "MOSAIC-II_16@CTIO" << "MOSAIC-III_4@KPNO_4m"
+                         << "PISCO@LCO" << "SAMI_2x2@SOAR" << "SOI@SOAR";
     if (multiChannelMultiExt.contains(instData.name)) ampInSeparateExt = true;
 
     if (instruments.contains(instData.name)) {
@@ -276,13 +278,30 @@ void Splitter::extractImagesFITS()
     // is identical to the 'chip' number used throughout the rest of this source code
     readPrimaryHeader();
 
+    // Check if the instrument matches the data
+    // This does NOT catch all inconsistent matches at this point (e.g. we are not comparing detector identification strings),
+    // and the INSTRUME keyword has not been determined for all pre-defined instruments yet. In the altter case, the test is skipped.
+    // And this works only for uncompressed data (INSTRUME keyword found in extension header of compressed data, which is not yet read at this point
+    QString instrument = "";
+    searchKeyValue(headerDictionary.value("INSTRUME"), instrument);
+    if (!checkInstrumentConsistency(instrument)) {
+        if (rawStatus == END_OF_FILE) rawStatus = 0;
+        fits_close_file(rawFptr, &rawStatus);
+        printCfitsioError(__func__, rawStatus);
+        return;
+    }
+
     // FORCE a beginning with the absolute first HDU. If I use 'movrel' then I'm not sure it is going to skip
     // over the first HDU if there is one. To be tested with suitable data. Then we could remove the 'movabs'
     // and make 'movrel' the first command inside 'while () {}'
     fits_movabs_hdu(rawFptr, 1, &hduType, &rawStatus);
     while (rawStatus != END_OF_FILE && successProcessing) {
+
         if (hduType == IMAGE_HDU) {
 
+            // for MMIRS, we only want to keep the first extension
+            if (instData.name.contains("MMIRS") && chip >= 1) break;
+
             // some multi-chip cams (FORS, etc) come with separate FITS files. For them, 'chip' would always be zero,
             // and thus the correct overscan regions etc not identified correctly.
             // Others such as GROND image simultaneously in different bandpasses on multiple detectors, but they show the
@@ -290,6 +309,11 @@ void Splitter::extractImagesFITS()
             // Hence this mapping
             int chipMapped = inferChipID(chip) - 1;   // same value as chip for normal 'MEF' files
 
+            // For HSC@SUBARU, drop the 8 focus CCDs
+            if (instData.name == "HSC@SUBARU" && chipMapped >= 104) break;
+            // HSC chips too difficult for astrometry
+            // "1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,22,23,24,29,30,31,32,37,38,39,40,45,46,47,48,53,54,55,56,61,62,63,64,69,70,71,72,77,78,79,84,85,86,87,88,89,90,91,92,93,94,95,96,97,98,99,100,101,102,103,104"
+
             // do we have an "image" (as compared to a data unit that is simply a nullptr)
             int naxis = -1;
             fits_get_img_dim(rawFptr, &naxis, &rawStatus);
@@ -308,6 +332,11 @@ void Splitter::extractImagesFITS()
 
             // OK, we have either a 2D image or a cube.
 
+            // Saturation handling
+            if (instData.type == "OPT") saturationValue = pow(2,16)-1.;
+            else saturationValue = pow(2,20)-1.;    // HAWAII-2RGs can on-chip coadd exposures, exceeding the usual 16-bit dynamic range
+            if (userSaturationValue != 0) saturationValue = userSaturationValue;
+
             // Build the header. Must clear before processing new chip
             headerTHELI.clear();
             readExtHeader();
@@ -324,7 +353,7 @@ void Splitter::extractImagesFITS()
             testGROND();                     // GROND data are an oddball and need to be tested here
             updateMEFpastingStatus(chip);    // Some cameras have readout channels in individual FITS extensions, implying skipping e.g. writeImage()
 
-            buildTheliHeaderFILTER();
+            buildTheliHeaderFILTER(chip);
             buildTheliHeaderWCS(chipMapped);
             buildTheliHeaderEXPTIME();
             buildTheliHeaderDATEOBS();  // must be done before MJD-OBS
@@ -334,11 +363,16 @@ void Splitter::extractImagesFITS()
             //            buildTheliHeaderGAIN(chipMapped);
             //            buildTheliHeader();
 
+            // Leave if this image is bad
+            if (!successProcessing) continue;
+
             // 2D image
             if (naxis == 2) {
                 getCurrentExtensionData();              // sets naxis1/2Raw, needed by everything below
                 getMultiportInformation(chipMapped);    // sets naxis1/2. Updates overscan and data sections for nonstandard multiport readouts
 
+                // overrideDetectorSections(chip);      // TESTING of a more autonomous procedure
+
                 // gains must be built after multiport chips are assembled
                 buildTheliHeaderGAIN(chipMapped);
                 buildTheliHeader();
@@ -349,6 +383,7 @@ void Splitter::extractImagesFITS()
                 //                cropDataSection(dataSection[chipMapped]);
                 pasteMultiportIlluminatedSections(chipMapped);
                 correctXtalk();             // Must maintain original detector geometry for x-talk correction, i.e. do before cropping. Must replace naxisi by naxisiRaw in xtalk methods.
+
                 correctNonlinearity(chipMapped);
                 convertToElectrons(chipMapped);
                 applyMask(chipMapped);
@@ -418,7 +453,7 @@ void Splitter::extractImagesFITS()
     if (rawStatus == END_OF_FILE) rawStatus = 0;
     fits_close_file(rawFptr, &rawStatus);
 
-    printCfitsioError("extractImagesFITS()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
 }
 
 // skip over bad detectors
@@ -456,6 +491,7 @@ bool Splitter::isImageUsable(int &chipMapped)
 int Splitter::inferChipID(int chip)
 {
     // Data from some cameras, such as SuprimeCam and FORS, come in separate FITS files instead of a MEF file.
+    // Multichannel imagers also need to be treated here
     // We need to identify the chip number correctly:
     int chipID = chip + 1;    // external counting starts with zero; 'chipID' is the number we return for most instruments
 
@@ -485,11 +521,20 @@ int Splitter::inferChipID(int chip)
         return chipID;
     }
 
+    else if (instData.name == "Direct_4k_SWOPE@LCO"
+             || instData.name.contains("LDSS3")) {
+        int value = 0;
+        searchKeyValue(QStringList() << "OPAMP", value);    // running from 1-2 (LDSS3) and from 1-4 (Direct_4k)
+        chipID = value;
+        return chipID;
+    }
+
     else if (instData.name == "SuprimeCam_200105-200807@SUBARU"
              || instData.name == "SuprimeCam_200101-200104@SUBARU"
-             || instData.name == "SuprimeCam_200808-201705@SUBARU") {
+             || instData.name == "SuprimeCam_200808-201705@SUBARU"
+             || instData.name == "HSC@SUBARU") {
         int value = 0;
-        searchKeyValue(QStringList() << "DET-ID", value);    // running from 0 to 9
+        searchKeyValue(QStringList() << "DET-ID", value);    // running from 0 to 9 (and 0 to 111 for HSC)
         chipID = value + 1;
         return chipID;
     }
@@ -508,6 +553,13 @@ int Splitter::inferChipID(int chip)
         return chipID;
     }
 
+    else if (instData.name.contains("IMACS")) {
+        int value = 0;
+        searchKeyValue(QStringList() << "CHIP", value);    // running from 1 to 8
+        chipID = value;
+        return chipID;
+    }
+
     else if (instData.name == "SPARTAN@SOAR") {
         int value = 0;
         searchKeyValue(QStringList() << "DETSERNO", value);    // detector serial number
@@ -522,7 +574,11 @@ int Splitter::inferChipID(int chip)
         return chipID;
     }
 
-    else if (instData.name == "GROND_OPT@MPGESO" || instData.name == "GROND_NIR@MPGESO") {
+    // Multichannel imagers: have just one (so far) detector number per channel, and the camera.ini specifies just 1 chip
+    // Otherwise chipID runs to values larger than 1 which crashes queries of the Mask class
+    else if (instData.name == "GROND_OPT@MPGESO"
+             || instData.name == "GROND_NIR@MPGESO") {
+        //             || instData.name == "PISCO@LCO") {            // must maintain chip id because of chip-dependent pasting
         // Simultaneous observations in multiple bands, but just a single detector per band
         return 1;   // (reduced by -1 in caller)
     }
@@ -543,6 +599,7 @@ void Splitter::getCurrentExtensionData()
     // Read the data block
     naxis1Raw = naxis[0];
     naxis2Raw = naxis[1];
+
     long nelements = naxis1Raw*naxis2Raw;
     float *buffer = new float[nelements];
     float nullval = 0.;
@@ -567,7 +624,7 @@ void Splitter::getCurrentExtensionData()
     delete [] buffer;
     buffer = nullptr;
 
-    printCfitsioError("getCurrentExtensionData()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
 }
 
 // UNUSED
@@ -609,7 +666,7 @@ void Splitter::getDataInFirstCubeSlice()
     buffer = nullptr;
     fpixel = nullptr;
 
-    printCfitsioError("getDataInFirstCubeSlice()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
 }
 
 void Splitter::getDataInCube()
@@ -643,7 +700,7 @@ void Splitter::getDataInCube()
     delete [] bufferAll;
     bufferAll = nullptr;
 
-    printCfitsioError("getDataInCube()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
 }
 
 void Splitter::sliceCube(long slice)
@@ -724,6 +781,21 @@ void Splitter::getDetectorSections()
     }
 }
 
+// TESTING
+void Splitter::overrideDetectorSections(int chip)
+{
+    if (instData.name == "MOSAIC-III@KPNO_4m"
+            || instData.name == "WFC@INT") {
+        multiportOverscanDirections << "vertical";
+        QVector<long> tmp = extractVerticesFromKeyword("BIASSEC");
+        overscanX[chip] << tmp.at(0) << tmp.at(1);
+        dataSection[chip] = extractVerticesFromKeyword("DATASEC");
+        dataSection[chip] = extractVerticesFromKeyword("TRIMSEC");
+        naxis1 = dataSection[chip].at(1) - dataSection[chip].at(0);
+        naxis2 = dataSection[chip].at(3) - dataSection[chip].at(2);
+    }
+}
+
 void Splitter::getNumberOfAmplifiers()
 {
     if (!successProcessing) return;
@@ -739,15 +811,30 @@ void Splitter::getNumberOfAmplifiers()
             rawStatus = 0;
         }
     }
-    if (instData.name == "SOI@SOAR") {
+    // 2 amps per detector
+    if (instData.name == "SOI@SOAR"
+            || instData.name == "DEIMOS_2AMP@KECK"
+            || instData.name == "LRIS_BLUE@KECK"
+            || instData.name == "LRIS_RED@KECK"
+            || instData.name == "MOSAIC-II_16@CTIO"
+            || instData.name == "PISCO@LCO") {
         numAmpPerChip = 2;
         rawStatus = 0;
     }
+    // 4 amps per detector (geometric layout doesn't matter here)
+    if (instData.name == "MOSAIC-III_4@KPNO_4m"
+            || instData.name == "SAMI_2x2@SOAR") {
+        numAmpPerChip = 4;
+        rawStatus = 0;
+    }
 
     // multiple readout channels in different FITS extensions
-    multiChannelMultiExt << "GMOS-N-HAM@GEMINI" <<  "GMOS-N-HAM_1x1@GEMINI"
+    multiChannelMultiExt << "DEIMOS_2AMP@KECK"
+                         << "GMOS-N-HAM@GEMINI" <<  "GMOS-N-HAM_1x1@GEMINI"
                          << "GMOS-S-HAM@GEMINI" << "GMOS-S-HAM_1x1@GEMINI"
-                         << "SOI@SOAR";
+                         << "LRIS_BLUE@KECK" << "LRIS_RED@KECK"
+                         << "MOSAIC-II_16@CTIO" << "MOSAIC-III_4@KPNO_4m"
+                         << "PISCO@LCO" << "SAMI_2x2@SOAR" << "SOI@SOAR";
     if (multiChannelMultiExt.contains(instData.name)) ampInSeparateExt = true;
 
     if (numAmpPerChip > 1 && ampInSeparateExt) {
@@ -805,6 +892,45 @@ void Splitter::writeImage(int chipMapped)
             if (chipMapped == 1) chipID = 1;
             if (chipMapped == 3) chipID = 2;
         }
+        if (instData.name == "DEIMOS_2AMP@KECK") {
+            if (chipMapped == 1) chipID = 1;
+            if (chipMapped == 3) chipID = 2;
+            if (chipMapped == 5) chipID = 3;
+            if (chipMapped == 7) chipID = 4;
+        }
+        if (instData.name == "LRIS_BLUE@KECK") {
+            if (chipMapped == 1) chipID = 1;
+            if (chipMapped == 3) chipID = 2;
+        }
+        if (instData.name == "LRIS_RED@KECK") {
+            if (chipMapped == 1) chipID = 1;
+            if (chipMapped == 3) chipID = 2;
+        }
+        if (instData.name == "MOSAIC-II_16@CTIO") {
+            if (chipMapped == 1) chipID = 1;
+            if (chipMapped == 3) chipID = 2;
+            if (chipMapped == 5) chipID = 3;
+            if (chipMapped == 7) chipID = 4;
+            if (chipMapped == 9) chipID = 5;
+            if (chipMapped == 11) chipID = 6;
+            if (chipMapped == 13) chipID = 7;
+            if (chipMapped == 15) chipID = 8;
+        }
+        if (instData.name == "PISCO@LCO") {        // multi-channel imager
+            if (chipMapped == 1) chipID = 1;
+            if (chipMapped == 3) chipID = 1;
+            if (chipMapped == 5) chipID = 1;
+            if (chipMapped == 7) chipID = 1;
+        }
+        if (instData.name == "MOSAIC-III_4@KPNO_4m") {
+            if (chipMapped == 3) chipID = 1;
+            if (chipMapped == 7) chipID = 2;
+            if (chipMapped == 11) chipID = 3;
+            if (chipMapped == 15) chipID = 4;
+        }
+        if (instData.name == "SAMI_2x2@SOAR") {
+            if (chipMapped == 3) chipID = 1;
+        }
         MEFpastingFinished = false;
     }
 
@@ -846,7 +972,7 @@ void Splitter::writeImage(int chipMapped)
     delete [] array;
     array = nullptr;
 
-    printCfitsioError("writeImage()", status);
+    printCfitsioError(__func__, status);
 }
 
 void Splitter::individualFixOutName(const int chipID)
@@ -859,14 +985,21 @@ void Splitter::individualFixOutName(const int chipID)
         uniqueID = uniqueID.simplified();
         individualFixDone = true;
     }
+    //    if (instData.name == "Direct_4k_SWOPE@LCO") {
+    //        test = searchKeyValue(QStringList() << "FILENAME", uniqueID);
+    //        uniqueID = uniqueID.remove(7,2);
+    //        individualFixDone = true;
+    //    }
     else if (instData.name.contains("MOIRCS")) {
         test = searchKeyValue(QStringList() << "EXP-ID", uniqueID);    // e.g. MCSE00012193
         individualFixDone = true;
     }
-    //    else if (instData.name == "GROND_OPT@MPGESO") {
     else if (instNameFromData == "GROND_OPT@MPGESO") {
         individualFixDone = true;
     }
+    else if (instData.name == "PISCO@LCO") {
+        individualFixDone = true;
+    }
 
     if (individualFixDone) {
         if (!test) {
@@ -875,13 +1008,18 @@ void Splitter::individualFixOutName(const int chipID)
             successProcessing = false;
         }
         else {
-            //if (instData.name == "GROND_OPT@MPGESO") {
             if (instNameFromData == "GROND_OPT@MPGESO") {
                 QString newPath = path+"_"+filter;
                 QDir newDir(newPath);
                 newDir.mkpath(newPath);
                 splitFileName = "!"+newPath+"/"+instData.shortName+"."+filter+"."+dateObsValue+"_"+QString::number(chipID)+"P.fits";
             }
+            else if (instData.name == "PISCO@LCO") {
+                QString newPath = path+"_"+filter;
+                QDir newDir(newPath);
+                newDir.mkpath(newPath);
+                splitFileName = "!"+newPath+"/"+instData.shortName+"."+filter+"."+dateObsValue+"_"+QString::number(chipID)+"P.fits";
+            }
             else {
                 splitFileName = "!"+path+"/"+instData.shortName+"."+filter+"."+uniqueID+"_"+QString::number(chipID)+"P.fits";
             }
@@ -970,13 +1108,12 @@ bool Splitter::individualFixWriteImage(int chipMapped)
             delete [] array;
             array = nullptr;
 
-            printCfitsioError("writeImage()", status);
+            printCfitsioError(__func__, status);
         }
         individualFixDone = true;
     }
 
-    //    if (instData.name == "GROND_NIR@MPGESO") {    // Write the three channels as separate FITS files
-    if (instNameFromData == "GROND_NIR@MPGESO") {    // Write the three channels as separate FITS files
+    if (instNameFromData == "GROND_NIR@MPGESO") {       // Write the three channels as separate FITS files
         for (int channel=0; channel<=2; ++channel) {
             fitsfile *fptr;
             int status = 0;
@@ -1052,7 +1189,7 @@ bool Splitter::individualFixWriteImage(int chipMapped)
             delete [] array;
             array = nullptr;
 
-            printCfitsioError("writeImage()", status);
+            printCfitsioError(__func__, status);
         }
         individualFixDone = true;
     }
@@ -1114,7 +1251,7 @@ void Splitter::writeImageSlice(int chip, long slice)
     delete [] array;
     array = nullptr;
 
-    printCfitsioError("writeImageSlice()", status);
+    printCfitsioError(__func__, status);
 }
 
 // UNUSED. Not sure whether I need the MyImages sorted
@@ -1175,7 +1312,7 @@ void Splitter::readExtHeader()
     char *fullheader = nullptr;
     int numHeaderKeys = 0;
     fits_hdr2str(rawFptr, TRUE, NULL, 0, &fullheader, &numHeaderKeys, &rawStatus);
-    printCfitsioError("readextheader()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
     if (rawStatus) return;
 
     fullExtHeaderString = QString::fromUtf8(fullheader);
@@ -1202,7 +1339,7 @@ void Splitter::readPrimaryHeader()
     char *fullheader = nullptr;
     int numHeaderKeys = 0;
     fits_hdr2str(rawFptr, TRUE, NULL, 0, &fullheader, &numHeaderKeys, &rawStatus);
-    printCfitsioError("readPrimaryHeader()", rawStatus);
+    printCfitsioError(__func__, rawStatus);
     if (rawStatus) return;
 
     fullPrimaryHeaderString = QString::fromUtf8(fullheader);
@@ -1224,15 +1361,29 @@ bool Splitter::checkCorrectMaskSize(const int chip)
 {
     long n_mask = mask->globalMask[chip].length();
     long n_data = dataCurrent.length();
-    if (n_mask > 0 && n_mask != n_data) {
+    // detectors with multi-amps (one amp per FITS extension): masked at a later stage (calibration)
+    if (n_mask > 0 && n_data > 0 && n_mask != n_data) {
         if (!maskSizeWarningShown) {
-            emit messageAvailable("Inconsistent image size detected between data and instrument configuration"
-                                  " (overscan and / or data section) in\n"+instData.nameFullPath, "error");
-            emit critical();
-            successProcessing = false;
-            maskSizeWarningShown = true;
+            emit messageAvailable(fileName + ": Inconsistent image size detected between data and instrument configuration"
+                                             " (overscan and / or data section) in\n"+instData.nameFullPath+
+                                  " \n. The image will not be processed, please remove it manually.", "warning");
+            emit warning();
+            //            successProcessing = false;
+            //            maskSizeWarningShown = true;
             return false;
         }
     }
     return true;
 }
+
+bool Splitter::checkInstrumentConsistency(QString foundInstrumentName)
+{
+    QString expectedInstrumentName = instrumentDictionary.value(instData.name);
+    if (!expectedInstrumentName.isEmpty() && expectedInstrumentName != foundInstrumentName) {
+        emit messageAvailable(fileName + ": Wrong instrument selected: Expected " + instData.name + ", but \"INSTRUME= " + foundInstrumentName+"\"", "error");
+        emit critical();
+        successProcessing = false;
+        return false;
+    }
+    return true;
+}
