@@ -69,6 +69,14 @@ void Splitter::buildTheliHeader()
         }
     }
 
+    // Append geographic coordinates
+    QString geolon = "GEOLON  = "+QString::number(instData.obslon, 'f', 4);
+    QString geolat = "GEOLAT  = "+QString::number(instData.obslat, 'f', 4);
+    geolon.resize(80, ' ');
+    geolat.resize(80, ' ');
+    headerTHELI.append(geolon);
+    headerTHELI.append(geolat);
+
     // Instrument-specific optional keys
     bool keyFoundOptional = true;
     QString instSpecificKey = "";
@@ -98,7 +106,9 @@ void Splitter::buildTheliHeaderMJDOBS()
 
     // List of instruments for which MJD-OBS is not reliable and should be constructed from DATE-OBS
     QStringList instruments = {"GSAOI@GEMINI", "GSAOI_CHIP1@GEMINI", "GSAOI_CHIP2@GEMINI", "GSAOI_CHIP3@GEMINI", "GSAOI_CHIP4@GEMINI",
-                               "FLAMINGOS2@GEMINI"};
+                               "FLAMINGOS2@GEMINI",
+                               "SAMI_2x2@SOAR"          // integer
+                              };
     if (instruments.contains(instData.name)) {
         if (!dateObsValue.isEmpty()) {
             mjdobsValue = dateobsToMJD();
@@ -140,6 +150,7 @@ void Splitter::buildTheliHeaderMJDOBS()
     }
 }
 
+// UNUSED
 void Splitter::buildTheliHeaderSATURATION()
 {
     QString card = "SATURATE= " + QString::number(saturationValue, 'f', 6) + " / Saturation level for this image";
@@ -169,25 +180,35 @@ void Splitter::buildTheliHeaderWCS(int chip)
 
 void Splitter::WCSbuildCRPIX(int chip)
 {
+    if (!successProcessing) return;
+
     // Exceptions. Return if successful.
     if (individualFixCRPIX(chip)) return;
 
     QStringList headerWCS;
 
-    // CRPIXi: Rely on instrument.ini (Todo: scan .ahead files directly for multi-chip cameras)
-    QString crpix1_card = "CRPIX1  = "+QString::number(instData.crpix1[chip]);
-    QString crpix2_card = "CRPIX2  = "+QString::number(instData.crpix2[chip]);
-    crpix1_card.resize(80, ' ');
-    crpix2_card.resize(80, ' ');
-    headerWCS.append(crpix1_card);
-    headerWCS.append(crpix2_card);
-
+    // Use dedicated lookup
+    if (instData.name == "HSC@SUBARU") {
+        searchKeyCRVAL("CRPIX1", headerDictionary.value("CRPIX1"), headerWCS);
+        searchKeyCRVAL("CRPIX2", headerDictionary.value("CRPIX2"), headerWCS);
+    }
+    else {
+        // CRPIXi: Rely on instrument.ini (Todo: scan .ahead files directly for multi-chip cameras)
+        QString crpix1_card = "CRPIX1  = "+QString::number(instData.crpix1[chip]);
+        QString crpix2_card = "CRPIX2  = "+QString::number(instData.crpix2[chip]);
+        crpix1_card.resize(80, ' ');
+        crpix2_card.resize(80, ' ');
+        headerWCS.append(crpix1_card);
+        headerWCS.append(crpix2_card);
+    }
     headerTHELI.append(headerWCS);
 }
 
 // Instrument dependent
 void Splitter::WCSbuildCRVAL()
 {
+    if (!successProcessing) return;
+
     // Exceptions. Return if successful.
     if (individualFixCRVAL()) return;
 
@@ -202,6 +223,8 @@ void Splitter::WCSbuildCRVAL()
 
 void Splitter::WCSbuildCTYPE()
 {
+    if (!successProcessing) return;
+
     QStringList headerWCS;
 
     // CTYPEi are fixed no matter what!
@@ -218,6 +241,8 @@ void Splitter::WCSbuildCTYPE()
 // Instrument dependent
 void Splitter::WCSbuildCDmatrix(int chip)
 {
+    if (!successProcessing) return;
+
     // Exceptions. Return if successful.
     if (individualFixCDmatrix(chip)) return;
 
@@ -237,17 +262,29 @@ void Splitter::WCSbuildCDmatrix(int chip)
 
     for (auto &wcsKey : wcsKeys) {
         bool keyFound = searchKey(wcsKey, headerDictionary.value(wcsKey), headerWCS);
-        // default values if failed
+
+        // fallback
+        double cd = 0.0;
         if (!keyFound && wcsKey == "CD1_1") {
-            bool found = searchKey("CD1_1", headerDictionary.value("CDELT1"), headerWCS);  // first argument, CD1_1, is taken to form the new header card. Value is taken from CDELT1
-            if (!found) fallback = "CD1_1   = "+QString::number(-1.*flipcd11*instData.pixscale/3600., 'g', 6);
+            bool success = CDfromCDELTandPC("CDELT1", "PC1_1", cd);
+            if (success) fallback = "CD1_1   = "+QString::number(cd, 'g', 6);
+            else fallback = "CD1_1   = "+QString::number(-1.*flipcd11*instData.pixscale/3600., 'g', 6);
         }
         if (!keyFound && wcsKey == "CD2_2") {
-            bool found = searchKey("CD2_2", headerDictionary.value("CDELT2"), headerWCS);
-            if (!found) fallback = "CD2_2   = "+QString::number(flipcd22*instData.pixscale/3600., 'g', 6);
+            bool success = CDfromCDELTandPC("CDELT2", "PC2_2", cd);
+            if (success) fallback = "CD2_2   = "+QString::number(cd, 'g', 6);
+            else fallback = "CD2_2   = "+QString::number(flipcd22*instData.pixscale/3600., 'g', 6);
+        }
+        if (!keyFound && wcsKey == "CD1_2") {
+            bool success = CDfromCDELTandPC("CDELT1", "PC1_2", cd);
+            if (success) fallback = "CD1_2   = "+QString::number(cd, 'g', 6);
+            else fallback = "CD1_2   = 0.0";
+        }
+        if (!keyFound && wcsKey == "CD2_1") {
+            bool success = CDfromCDELTandPC("CDELT2", "PC2_1", cd);
+            if (success) fallback = "CD2_1   = "+QString::number(cd, 'g', 6);
+            else fallback = "CD2_1   = 0.0";
         }
-        if (!keyFound && wcsKey == "CD1_2") fallback = "CD1_2   = 0.0";
-        if (!keyFound && wcsKey == "CD2_1") fallback = "CD2_1   = 0.0";
 
         if (!keyFound) {
             if (*verbosity > 1) emit messageAvailable(fileName + " : Could not determine keyword: "+wcsKey+", using default: "+fallback, "ignore");
@@ -259,8 +296,32 @@ void Splitter::WCSbuildCDmatrix(int chip)
     headerTHELI.append(headerWCS);
 }
 
+bool Splitter::CDfromCDELTandPC(const QString cdeltstr, const QString pcstr, double &cd)
+{
+    double cdelt = -1.0;
+    double pc = -1.0;
+    bool cdtest = searchKeyValue(headerDictionary.value(cdeltstr), cdelt);
+    bool pctest = searchKeyValue(headerDictionary.value(pcstr), pc);
+    if (cdtest && pctest) {
+        cd = cdelt*pc;
+        if (fabs(cd) > 0.5) cd *= instData.pixscale/3600.;   // recover plate scale if not available
+        return true;
+    }
+    else if (cdtest && !pctest) {
+        cd = cdelt;
+        if (fabs(cd) > 0.5) cd *= instData.pixscale/3600.;
+        return true;
+    }
+    else {
+        cd = 0.;
+        return false;
+    }
+}
+
 void Splitter::WCSbuildRADESYS()
 {
+    if (!successProcessing) return;
+
     QStringList headerWCS;
     QString wcsKey = "RADESYS";
     bool keyFound = searchKey(wcsKey, headerDictionary.value(wcsKey), headerWCS);
@@ -275,6 +336,10 @@ void Splitter::WCSbuildRADESYS()
 
 void Splitter::WCSbuildEQUINOX()
 {
+    if (!successProcessing) return;
+
+    if (individualFixEQUINOX()) return;
+
     QStringList headerWCS;
     QString wcsKey = "EQUINOX";
     bool keyFound = searchKey(wcsKey, headerDictionary.value(wcsKey), headerWCS);
@@ -287,8 +352,33 @@ void Splitter::WCSbuildEQUINOX()
     headerTHELI.append(headerWCS);
 }
 
+bool Splitter::individualFixEQUINOX()
+{
+    if (!successProcessing) return false;
+
+    bool individualFixDone = false;
+
+    QStringList headerWCS;
+    QString equinoxCard = "";
+
+    if (instData.name == "SITe3_SWOPE@LCO") {
+        equinoxCard = equinoxCard = "EQUINOX = 2000.0";            // SITe3_SWOPE@LCO has EPOCH written to EQUINOX, which makes scamp fail since the coords are in J2000.
+        individualFixDone = true;
+    }
+
+    if (individualFixDone) {
+        equinoxCard.resize(80, ' ');
+        headerWCS.append(equinoxCard);
+        headerTHELI.append(headerWCS);
+    }
+
+    return individualFixDone;
+}
+
 bool Splitter::individualFixCRVAL()
 {
+    if (!successProcessing) return false;
+
     bool individualFixDone = false;
 
     QStringList headerWCS;
@@ -298,7 +388,7 @@ bool Splitter::individualFixCRVAL()
     QString crval2 = "";
 
     // List of instruments that we have to consider
-    QStringList list = {"WFC@INT", "SUSI1@NTT"};
+    QStringList list = {"WFC@INT", "SUSI1@NTT", "PISCO@LCO"};
 
     // Leave if no individual fix is required.
     if (!list.contains(instData.name)) return false;
@@ -333,6 +423,15 @@ bool Splitter::individualFixCRVAL()
         crval2_card = "CRVAL2  = "+crval2;
         individualFixDone = true;
     }
+    if (instData.name == "PISCO@LCO") {
+        searchKeyValue(QStringList() << "TELRA", crval1);    // CRVALij in FITS extensions are highly inaccurate
+        searchKeyValue(QStringList() << "TELDC", crval2);
+        if (crval1.contains(":")) crval1 = hmsToDecimal(crval1);
+        if (crval2.contains(":")) crval2 = dmsToDecimal(crval2);
+        crval1_card = "CRVAL1  = "+crval1;
+        crval2_card = "CRVAL2  = "+crval2;
+        individualFixDone = true;
+    }
 
     if (individualFixDone) {
         crval1_card.resize(80, ' ');
@@ -347,6 +446,8 @@ bool Splitter::individualFixCRVAL()
 
 bool Splitter::individualFixCRPIX(int chip)
 {
+    if (!successProcessing) return false;
+
     bool individualFixDone = false;
 
     QStringList headerWCS;
@@ -391,6 +492,78 @@ bool Splitter::individualFixCRPIX(int chip)
         individualFixDone = true;
     }
 
+    if (instData.name == "DEIMOS_2AMP@KECK") {
+        if (chip == 1) crpix1_card = "CRPIX1  = 4253";
+        if (chip == 3) crpix1_card = "CRPIX1  = 2097";
+        if (chip == 5) crpix1_card = "CRPIX1  = -53";
+        if (chip == 7) crpix1_card = "CRPIX1  = -2200";
+        crpix2_card = "CRPIX2  = 1301";
+        individualFixDone = true;
+    }
+
+    if (instData.name == "LRIS_BLUE@KECK") {
+        if (chip == 1) crpix1_card = "CRPIX1  = 2099";
+        if (chip == 3) crpix1_card = "CRPIX1  = -52";
+        crpix2_card = "CRPIX2  = 2052";
+        individualFixDone = true;
+    }
+
+    if (instData.name == "LRIS_RED@KECK") {
+        if (chip == 1) crpix1_card = "CRPIX1  = -126";
+        if (chip == 3) crpix1_card = "CRPIX1  = 1835";
+        crpix2_card = "CRPIX2  = 1260";
+        individualFixDone = true;
+    }
+
+    if (instData.name == "MOSAIC-II_16@CTIO") {
+        if (chip == 1) crpix1_card = "CRPIX1  = 4219";
+        if (chip == 3) crpix1_card = "CRPIX1  = 2078";
+        if (chip == 5) crpix1_card = "CRPIX1  = -33";
+        if (chip == 7) crpix1_card = "CRPIX1  = -2166";
+        if (chip == 9) crpix1_card = "CRPIX1  = 4221";
+        if (chip == 11) crpix1_card = "CRPIX1  = 2081";
+        if (chip == 13) crpix1_card = "CRPIX1  = -31";
+        if (chip == 15) crpix1_card = "CRPIX1  = -2169";
+        if (chip == 1) crpix2_card = "CRPIX2  = 4148";
+        if (chip == 3) crpix2_card = "CRPIX2  = 4121";
+        if (chip == 5) crpix2_card = "CRPIX2  = 4119";
+        if (chip == 7) crpix2_card = "CRPIX2  = 4136";
+        if (chip == 9) crpix2_card = "CRPIX2  = -34";
+        if (chip == 11) crpix2_card = "CRPIX2  = -24";
+        if (chip == 13) crpix2_card = "CRPIX2  = -26";
+        if (chip == 15) crpix2_card = "CRPIX2  = -56";
+        individualFixDone = true;
+    }
+    if (instData.name == "PISCO@LCO") {
+        if (chip == 1) crpix1_card = "CRPIX1  = 1500";
+        if (chip == 3) crpix1_card = "CRPIX1  = 1500";
+        if (chip == 5) crpix1_card = "CRPIX1  = 1500";
+        if (chip == 7) crpix1_card = "CRPIX1  = 1500";
+        if (chip == 1) crpix2_card = "CRPIX2  = 3000";
+        if (chip == 3) crpix2_card = "CRPIX2  = 3000";
+        if (chip == 5) crpix2_card = "CRPIX2  = 3000";
+        if (chip == 7) crpix2_card = "CRPIX2  = 3000";
+        individualFixDone = true;
+    }
+    if (instData.name == "MOSAIC-III_4@KPNO_4m") {
+        if (chip == 3) crpix1_card = "CRPIX1  = 4219";
+        if (chip == 7) crpix1_card = "CRPIX1  = 2078";
+        if (chip == 11) crpix1_card = "CRPIX1  = -33";
+        if (chip == 15) crpix1_card = "CRPIX1  = -2166";
+        if (chip == 3) crpix2_card = "CRPIX2  = 4148";
+        if (chip == 7) crpix2_card = "CRPIX2  = 4121";
+        if (chip == 11) crpix2_card = "CRPIX2  = 4119";
+        if (chip == 15) crpix2_card = "CRPIX2  = 4136";
+        individualFixDone = true;
+    }
+    if (instData.name == "SAMI_2x2@SOAR") {
+        if (chip == 3) {
+            crpix1_card = "CRPIX1  = 1024";
+            crpix1_card = "CRPIX1  = 1024";
+        }
+        individualFixDone = true;
+    }
+
     // Append only when all channels of one chip have been read (i.e., 'individualFixDone' == true)
     if (individualFixDone) {
         crpix1_card.resize(80, ' ');
@@ -405,15 +578,78 @@ bool Splitter::individualFixCRPIX(int chip)
 
 bool Splitter::individualFixCDmatrix(int chip)
 {
+    if (!successProcessing) return false;
+
     bool individualFixDone = false;
 
     QStringList headerWCS;
 
+    double cd11 = 0.;
+    double cd12 = 0.;
+    double cd21 = 0.;
+    double cd22 = 0.;
+
     QString cd11_card = "";
     QString cd12_card = "";
     QString cd21_card = "";
     QString cd22_card = "";
 
+    if (instData.name == "Direct_2k_DUPONT@LCO") {
+        cd11_card = "CD1_1   =  0.0";
+        cd12_card = "CD1_2   =  -7.2273e-5";
+        cd21_card = "CD2_1   =  -7.2273e-5";
+        cd22_card = "CD2_2   =  0.0";
+        individualFixDone = true;
+    }
+    if (instData.name == "SITe3_SWOPE@LCO") {
+        cd11_card = "CD1_1   =  -1.2083e-4";
+        cd12_card = "CD1_2   =  0.0";
+        cd21_card = "CD2_1   =  0.0";
+        cd22_card = "CD2_2   =  -1.2083e-4";
+        individualFixDone = true;
+    }
+    if (instData.name == "Direct_4k_SWOPE@LCO") {
+        if (chip == 0) {
+            cd11_card = "CD1_1   =  8.839493716E-07";
+            cd12_card = "CD1_2   =  -1.209379295E-04";
+            cd21_card = "CD2_1   =  -1.209379295E-04";
+            cd22_card = "CD2_2   =  -8.839493716E-07";
+        }
+        if (chip == 1) {
+            cd11_card = "CD1_1   =  -8.861927404E-07";
+            cd12_card = "CD1_2   =  -1.209344868E-04";
+            cd21_card = "CD2_1   =  1.209344868E-04";
+            cd22_card = "CD2_2   =  -8.861927404E-07";
+        }
+        if (chip == 2) {
+            cd11_card = "CD1_1   =  -7.788683939E-07";
+            cd12_card = "CD1_2   =  1.211132665E-04";
+            cd21_card = "CD2_1   =  1.211132665E-04";
+            cd22_card = "CD2_2   =  7.788683939E-07";
+        }
+        if (chip == 3) {
+            cd11_card = "CD1_1   =  8.755109826E-07";
+            cd12_card = "CD1_2   =  1.208828016E-04";
+            cd21_card = "CD2_1   =  -1.208828016E-04";
+            cd22_card = "CD2_2   =  8.755109826E-07";
+        }
+        individualFixDone = true;
+    }
+    if (instData.name == "LDSS3_from201402@LCO") {
+        if (chip == 0) {
+            cd11_card = "CD1_1   =  -5.213e-5";
+            cd12_card = "CD1_2   =  0.0";
+            cd21_card = "CD2_1   =  0.0";
+            cd22_card = "CD2_2   =  5.213e-5";
+        }
+        if (chip == 1) {
+            cd11_card = "CD1_1   =  5.230e-5";
+            cd12_card = "CD1_2   =  0.0";
+            cd21_card = "CD2_1   =  0.0";
+            cd22_card = "CD2_2   =  5.230e-5";
+        }
+        individualFixDone = true;
+    }
     if (instData.name == "WFI@MPGESO") {
         cd11_card = "CD1_1   =  -6.611e-5";
         cd12_card = "CD1_2   =  0.0";
@@ -421,6 +657,34 @@ bool Splitter::individualFixCDmatrix(int chip)
         cd22_card = "CD2_2   =  6.611e-5";
         individualFixDone = true;
     }
+    if (instData.name == "DEIMOS_1AMP@KECK" || instData.name == "DEIMOS_2AMP@KECK") {
+        cd11_card = "CD1_1   =  0.0";
+        cd12_card = "CD1_2   =  3.28000e-5";
+        cd21_card = "CD2_1   =  3.28000e-5";
+        cd22_card = "CD2_2   =  0.0";
+        individualFixDone = true;
+    }
+    if (instData.name == "LRIS_BLUE@KECK") {
+        cd11_card = "CD1_1   =  0.0";
+        cd12_card = "CD1_2   =  -3.76e-5";
+        cd21_card = "CD2_1   =  3.76e-5";
+        cd22_card = "CD2_2   =  0.0";
+        individualFixDone = true;
+    }
+    if (instData.name == "LRIS_RED@KECK") {
+        cd11_card = "CD1_1   =  -3.76e-5";
+        cd12_card = "CD1_2   =  0.0";
+        cd21_card = "CD2_1   =  0.0";
+        cd22_card = "CD2_2   =  -3.76e-5";
+        individualFixDone = true;
+    }
+    if (instData.name == "ESI@KECK") {
+        cd11_card = "CD1_1   =  -4.28333e-5";
+        cd12_card = "CD1_2   =  0.0";
+        cd21_card = "CD2_1   =  0.0";
+        cd22_card = "CD2_2   =  -4.28333e-5";
+        individualFixDone = true;
+    }
     if (instData.name.contains("WFI_2x2") && instData.name.contains("MPGESO") ) {
         cd11_card = "CD1_1   =  -1.322e-4";
         cd12_card = "CD1_2   =  0.0";
@@ -461,10 +725,10 @@ bool Splitter::individualFixCDmatrix(int chip)
             emit warning();
             positionAngle = 0.0;
         }
-        double cd11 = -1.*instData.pixscale / 3600.;
-        double cd12 = 0.0;
-        double cd21 = 0.0;
-        double cd22 = instData.pixscale / 3600.;
+        cd11 = -1.*instData.pixscale / 3600.;
+        cd12 = 0.0;
+        cd21 = 0.0;
+        cd22 = instData.pixscale / 3600.;
         rotateCDmatrix(cd11, cd12, cd21, cd22, positionAngle);
         cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
         cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
@@ -472,16 +736,36 @@ bool Splitter::individualFixCDmatrix(int chip)
         cd22_card = "CD2_2   =  "+QString::number(cd22, 'g', 6);
         individualFixDone = true;
     }
+    if (instData.name.contains("IMACS")) {     // IMACS has no CD matrix in the header
+        if (!searchKeyValue(QStringList() << "ROTATORE", positionAngle)) {
+            emit messageAvailable(name + " : Could not find ROTANGLE keyword, set to zero! CD matrix might have wrong orientation.", "warning");
+            emit warning();
+            positionAngle = 0.0;
+        }
+
+        cd11 = instData.pixscale / 3600.;
+        cd12 = 0.;
+        cd21 = 0.;
+        cd22 = -instData.pixscale / 3600.;
+        // Chips 1-4 are rotated by 180 degrees
+        if (chip >= 4) rotateCDmatrix(cd11, cd12, cd21, cd22, positionAngle);
+        else rotateCDmatrix(cd11, cd12, cd21, cd22, positionAngle+180.);
+        cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
+        cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
+        cd21_card = "CD2_1   =  "+QString::number(cd21, 'g', 6);
+        cd22_card = "CD2_2   =  "+QString::number(cd22, 'g', 6);
+        individualFixDone = true;
+    }
     if (instData.name == "LIRIS@WHT" || instData.name == "LIRIS_POL@WHT") {     // LIRIS has no CD matrix in the header
         if (!searchKeyValue(QStringList() << "ROTSKYPA", positionAngle)) {
             emit messageAvailable(name + " : Could not find ROTSKYPA keyword, set to zero! CD matrix might have wrong orientation.", "warning");
             emit warning();
             positionAngle = 0.0;
         }
-        double cd11 = -1.*instData.pixscale / 3600.;
-        double cd12 = 0.0;
-        double cd21 = 0.0;
-        double cd22 = instData.pixscale / 3600.;
+        cd11 = -1.*instData.pixscale / 3600.;
+        cd12 = 0.0;
+        cd21 = 0.0;
+        cd22 = instData.pixscale / 3600.;
         rotateCDmatrix(cd11, cd12, cd21, cd22, positionAngle);
         cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
         cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
@@ -490,21 +774,20 @@ bool Splitter::individualFixCDmatrix(int chip)
         individualFixDone = true;
     }
     if (instData.name == "SUSI1@NTT") {
-        double cd11 = -3.611e-5;
-        double cd12 = 0.;
-        double cd21 = 0.;
-        double cd22 = 3.611e-5;
+        cd11 = -3.611e-5;
+        cd12 = 0.;
+        cd21 = 0.;
+        cd22 = 3.611e-5;
         cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
         cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
         cd21_card = "CD2_1   =  "+QString::number(cd21, 'g', 6);
         cd22_card = "CD2_2   =  "+QString::number(cd22, 'g', 6);
         individualFixDone = true;
     }
-    if (instNameFromData == "GROND_OPT@MPGESO") {                 // GROND optical data has both the NIR and OPT CD matrices in the header.
-        double cd11 = 0.;                                         // With the current scheme, the NIR matrix in the HDU gets picked over
-        double cd12 = 0.;                                         // OPT matrix in the extension
-        double cd21 = 0.;
-        double cd22 = 0.;
+    if (instNameFromData == "GROND_OPT@MPGESO") {
+        // GROND optical data has both the NIR and OPT CD matrices in the header.
+        // With the current scheme, the NIR matrix in the HDU gets picked over
+        // OPT matrix in the extension
         searchKeyValue(QStringList() << "CD1_1", cd11);
         searchKeyValue(QStringList() << "CD1_2", cd12);
         searchKeyValue(QStringList() << "CD2_1", cd21);
@@ -516,10 +799,6 @@ bool Splitter::individualFixCDmatrix(int chip)
         individualFixDone = true;
     }
     if (instNameFromData == "GROND_NIR@MPGESO") {                 // just double tapping
-        double cd11 = 0.;
-        double cd12 = 0.;
-        double cd21 = 0.;
-        double cd22 = 0.;
         searchKeyValue(QStringList() << "J_CD1_1", cd11);
         searchKeyValue(QStringList() << "J_CD1_2", cd12);
         searchKeyValue(QStringList() << "J_CD2_1", cd21);
@@ -531,10 +810,32 @@ bool Splitter::individualFixCDmatrix(int chip)
         individualFixDone = true;
     }
     if (instData.name == "PFC_old@WHT") {
-        double cd11 = 6.55e-5;
-        double cd12 = 5.0e-7;
-        double cd21 = 5.0e-7;
-        double cd22 = -6.55e-5;
+        cd11 = 6.55e-5;
+        cd12 = 5.0e-7;
+        cd21 = 5.0e-7;
+        cd22 = -6.55e-5;
+        cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
+        cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
+        cd21_card = "CD2_1   =  "+QString::number(cd21, 'g', 6);
+        cd22_card = "CD2_2   =  "+QString::number(cd22, 'g', 6);
+        individualFixDone = true;
+    }
+    if (instData.name == "PISCO@LCO") {
+        cd11 = -instData.pixscale / 3600.;
+        cd12 = 0.;
+        cd21 = 0.;
+        cd22 = -instData.pixscale / 3600.;
+        // headers appear unreliable (pixel scale wrong by a factor of two)
+        /*
+        searchKeyValue(QStringList() << "CD1_1", cd11);
+        searchKeyValue(QStringList() << "CD1_2", cd12);
+        searchKeyValue(QStringList() << "CD2_1", cd21);
+        searchKeyValue(QStringList() << "CD2_2", cd22);
+        // readout amplifiers are flipped, depending on chip
+        if (chip == 1 || chip == 2) {cd11 *= -1.; cd21 *= -1.;}
+        if (chip == 4 || chip == 7) {cd12 *= -1.; cd22 *= -1.;}
+        if (chip == 5 || chip == 6) {cd11 *= -1.; cd21 *= -1.; cd12 *= -1.; cd22 *= -1.;}
+        */
         cd11_card = "CD1_1   =  "+QString::number(cd11, 'g', 6);
         cd12_card = "CD1_2   =  "+QString::number(cd12, 'g', 6);
         cd21_card = "CD2_1   =  "+QString::number(cd21, 'g', 6);
@@ -644,8 +945,10 @@ void Splitter::buildTheliHeaderDATEOBS()
 {
     if (!successProcessing) return;
 
+    if (individualFixDATEOBS()) return;
+
     bool found = false;
-    // Lopp over all possible dateobs keywords, and break once we found one that is valid.
+    // Loop over all possible dateobs keywords, and break once we found one that is valid.
     // This is different from the general strategy to go over all possible keyword variants and take the first one that exists
     for (auto &keyword : headerDictionary.value("DATE-OBS")) {
         dateObsValue = "";
@@ -667,6 +970,7 @@ void Splitter::buildTheliHeaderDATEOBS()
         bool foundDATE = searchKeyValue(headerDictionary.value("DATE"), dateValue);
         bool foundTIME = searchKeyValue(headerDictionary.value("TIME"), timeValue);
         if (foundDATE && foundTIME) {
+            if (timeValue.contains(" ")) timeValue.replace(" ",":");         // E.g. SITe_SWOPE@LCO has a time stamp without colons
             if (dateValue.contains("-") && timeValue.contains(":")) {
                 dateObsValue = dateValue+"T"+timeValue;
             }
@@ -691,6 +995,53 @@ void Splitter::buildTheliHeaderDATEOBS()
     headerTHELI.append(card);
 }
 
+bool Splitter::individualFixDATEOBS()
+{
+    bool individualFixDone = false;
+
+    if (instData.name == "WHIRC@WIYN") {
+        double mjdValue = 0;
+        bool foundMJDOBS = searchKeyValue(headerDictionary.value("MJD-OBS"), mjdValue);
+        // DATE-OBS has the start time of a sequence
+        if (foundMJDOBS) {
+            dateObsValue = mjdobsToDATEOBS(mjdValue);
+            individualFixDone = true;
+        }
+    }
+
+    if (instData.name == "PISCO@LCO") {   // wrong order of DATE and DATEOBS keyword
+        QString dateValue;
+        QString timeValue;
+        bool foundDATE = searchKeyValue(QStringList() << "DATEOBS", dateValue);
+        bool foundTIME = searchKeyValue(QStringList() << "TELUT", timeValue);
+        if (foundDATE && foundTIME) {
+            if (dateValue.contains("-") && timeValue.contains(":")) {
+                dateObsValue = dateValue+"T"+timeValue;
+            }
+            else {
+                // Construct a unique dummy DATE-OBS keyword, incremented by 0.1 seconds.
+#pragma omp critical
+                {
+                    *dateObsIncrementor += 0.1;
+                    QString timeStamp = decimalSecondsToHms(*dateObsIncrementor);
+                    dateObsValue = "2020-01-01T"+timeStamp;
+                }
+                emit messageAvailable(fileName + " : Could not determine keyword: DATE-OBS, set to "+dateObsValue, "warning");
+                emit warning();
+            }
+        }
+        individualFixDone = true;
+    }
+
+    if (individualFixDone) {
+        QString card = "DATE-OBS= '"+dateObsValue+"'";
+        card.resize(80, ' ');
+        headerTHELI.append(card);
+    }
+
+    return individualFixDone;
+}
+
 // Build the GAIN keyword
 void Splitter::buildTheliHeaderGAIN(int chip)
 {
@@ -724,11 +1075,14 @@ void Splitter::buildTheliHeaderGAIN(int chip)
     headerTHELI.append(card1);
     headerTHELI.append(card2);
 
+    gainForSaturation = chipGain;
     gain[chip] = chipGain;   // used to convert the pixel data from ADU to electrons
 }
 
 bool Splitter::individualFixGAIN(int chip)
 {
+    if (!successProcessing) return false;
+
     bool individualFixDone = false;
 
     chipGain = 1.0;
@@ -739,6 +1093,22 @@ bool Splitter::individualFixGAIN(int chip)
         if (chip == 3) chipGain = 2.110;
         individualFixDone = true;
     }
+    if (instData.name == "SOFI@NTT") {           // https://www.eso.org/sci/facilities/lasilla/instruments/sofi/inst/setup/Detector_characteristic.html
+        chipGain = 5.3;
+        individualFixDone = true;
+    }
+    if (instData.name == "NEWFIRM@KPNO_4m") {    // https://www.noao.edu/ets/newfirm/documents/ORION_SCA_lab_tests_final.pdf
+        chipGain = 7.6;                          // same for all 4 chips
+        individualFixDone = true;
+    }
+    if (instData.name == "HDI@KPNO_0.9m") {      // https://www.noao.edu/0.9m/observe/hdi/hdi_manual.html
+        chipGain = 1.3;
+        individualFixDone = true;
+    }
+    if (instData.name == "WHIRC@WIYN") {         // https://www.noao.edu/kpno/manuals/whirc/whirc.user.html
+        chipGain = 3.4;
+        individualFixDone = true;
+    }
     else if (instData.name == "NIRI@GEMINI") {    // https://www.gemini.edu/sciops/instruments/niri/imaging/detector-array
         chipGain = 12.3;                          // No gain keyword in FITS header
         individualFixDone = true;
@@ -767,15 +1137,39 @@ bool Splitter::individualFixGAIN(int chip)
         if (chip == 7) chipGain = 2.03;
         individualFixDone = true;
     }
-    else if (instData.name == "SuprimeCam_200808-201705@SUBARU") {
+    else if (instData.name == "DEIMOS_1AMP@KECK") {
+        if (chip == 0) chipGain = 1.206;
+        if (chip == 1) chipGain = 1.200;
+        if (chip == 2) chipGain = 1.167;
+        if (chip == 3) chipGain = 1.217;
+        individualFixDone = true;
+    }
+    // multi-amp cameras where the gain is NOT available in the header
+    else if (instData.name == "DEIMOS_2AMP@KECK") {  // https://www2.keck.hawaii.edu/inst/obsdata/inst/deimos/www/detector_data/deimos_detector_data.html
+        if (chip == 0) chipGain = 1.206;
+        if (chip == 1) chipGain = 1.221;
+        if (chip == 2) chipGain = 1.200;
+        if (chip == 3) chipGain = 1.188;
+        if (chip == 4) chipGain = 1.167;
+        if (chip == 5) chipGain = 1.250;
+        if (chip == 6) chipGain = 1.217;
+        if (chip == 7) chipGain = 1.228;
         chipGain = harmonicGain(multiportGains);
         individualFixDone = true;
     }
-    else if (instData.name.contains("GMOS-N-HAM") || instData.name.contains("GMOS-S-HAM")) {
-        chipGain = harmonicGain(multiportGains);
-        individualFixDone = true;
-    }
-    else if (instData.name.contains("SOI@SOAR")) {
+    // multi-amp cameras where the gain is available in the header
+    else if (instData.name == "SuprimeCam_200808-201705@SUBARU"
+             || instData.name == "HSC@SUBARU"
+             || instData.name.contains("GMOS-N-HAM")
+             || instData.name.contains("GMOS-S-HAM")
+             || instData.name.contains("SAMI")          // have not added yet 1x1 confog
+             || instData.name.contains("SOI@SOAR")
+             || instData.name == "LRIS_BLUE@KECK"
+             || instData.name == "LRIS_RED@KECK"
+             || instData.name == "MOSAIC-II_16@CTIO"
+             || instData.name == "MOSAIC-III_4@KPNO_4m"
+             || instData.name == "PISCO@LCO"
+             ) {
         chipGain = harmonicGain(multiportGains);
         individualFixDone = true;
     }
@@ -813,12 +1207,27 @@ bool Splitter::individualFixGAIN(int chip)
         headerTHELI.append(card1);
         headerTHELI.append(card2);
 
-        if (instData.name.contains("GMOS-N-HAM") || instData.name.contains("GMOS-S-HAM")) {
-            gain[chip/4] = chipGain;
+        if (instData.name.contains("GMOS-N-HAM")
+                || instData.name.contains("GMOS-S-HAM")
+                || instData.name.contains("SAMI")
+                || instData.name.contains("SOI@SOAR")
+                || instData.name == "LRIS_BLUE@KECK"
+                || instData.name == "LRIS_RED@KECK"
+                || instData.name == "MOSAIC-II_16@CTIO"
+                || instData.name == "MOSAIC-III_4@KPNO_4m"
+                ) {
+            gain[chip/numAmpPerChip] = chipGain;
+        }
+        // multichannel imagers have less chips than gain.length() ('chip' counts higher than gain.length() )
+        else if (instData.name == "PISCO@LCO") {
+            gain[0] = chipGain;
+            gain[1] = chipGain;
         }
         else {
-            gain[chip] = chipGain;        // not applied for e.g. SuprimeCam_200808-201705
+            gain[chip] = chipGain;        // not applied for e.g. SuprimeCam_200808-201705 and the others (why?)
         }
+
+        gainForSaturation = chipGain;
     }
 
     return individualFixDone;
@@ -852,8 +1261,11 @@ void Splitter::buildTheliHeaderAIRMASS()
     }
 }
 
-void Splitter::buildTheliHeaderFILTER()
+void Splitter::buildTheliHeaderFILTER(int chip)
 {
+    // Exceptions. Return if successful.
+    if (individualFixFILTER(chip)) return;
+
     QStringList filterKeywordList;
     QStringList possibleKeyNames = headerDictionary.value("FILTER");
 
@@ -896,13 +1308,15 @@ void Splitter::buildTheliHeaderFILTER()
                             || filterName.contains("clr", Qt::CaseInsensitive)
                             || filterName.contains("csl", Qt::CaseInsensitive)          // MOIRCS
                             || filterName.contains("hole", Qt::CaseInsensitive)         // MOIRCS
+                            || filterName.contains("unavailable", Qt::CaseInsensitive)  // SAMI
                             || filterName.contains("open", Qt::CaseInsensitive)) {
                         clearFound = true;
                         continue;
                     }
-                    // Skip if filter name suggests that the slot was empty
+                    // Skip if filter name suggests that a dark was taken
                     if (filterName.contains("dark", Qt::CaseInsensitive)
-                            || filterName.contains("close", Qt::CaseInsensitive)) {
+                            || filterName.contains("close", Qt::CaseInsensitive)
+                            || filterName.contains("blocked", Qt::CaseInsensitive)) {
                         darkFound = true;
                         continue;
                     }
@@ -936,6 +1350,43 @@ void Splitter::buildTheliHeaderFILTER()
     headerTHELI.append(filterCard);
 }
 
+bool Splitter::individualFixFILTER(int chip)
+{
+    if (!successProcessing) return false;
+
+    bool individualFixDone = false;
+
+    QString filterCard = "";
+
+    if (instData.name == "PISCO@LCO") {
+        if (chip == 0 || chip == 1) filter = "g";
+        if (chip == 2 || chip == 3) filter = "r";
+        if (chip == 4 || chip == 5) filter = "i";
+        if (chip == 6 || chip == 7) filter = "z";
+        filterCard = "FILTER  = '"+filter+"'";
+        individualFixDone = true;
+    }
+
+    if (instData.name == "LRIS_BLUE@KECK") {
+        searchKeyValue(QStringList() << "BLUFILT", filter);
+        filterCard = "FILTER  = '"+filter+"'";
+        individualFixDone = true;
+    }
+
+    if (instData.name == "LRIS_RED@KECK") {
+        searchKeyValue(QStringList() << "REDFILT", filter);
+        filterCard = "FILTER  = '"+filter+"'";
+        individualFixDone = true;
+    }
+
+    if (individualFixDone) {
+        filterCard.resize(80, ' ');
+        headerTHELI.append(filterCard);
+    }
+
+    return individualFixDone;
+}
+
 bool Splitter::checkFormatDATEOBS()
 {
     // dateobs format: YYYY-MM-DDTHH:MM:SS.sss
@@ -958,6 +1409,34 @@ bool Splitter::checkFormatDATEOBS()
     return true;
 }
 
+QString Splitter::mjdobsToDATEOBS(double mjd)
+{
+    // Code taken and adjusted from:
+    // https://api.kde.org/4.12-api/kdeedu-apidocs/marble/html/astrolib_8cpp_source.html
+
+    // This program is free software licensed under the GNU LGPL. You can
+    // find a copy of this license in LICENSE.txt in the top directory of
+    // the source code.
+    //
+    // Copyright : Gerhard HOLTKAMP          11-JAN-2012
+
+    long jd0 = long(mjd + 2400001.0);
+    long b = long ((jd0 - 1867216.25) / 36524.25);     // Gregorian calendar
+    long c = jd0 + b - (b/4) + 1525;                   // Gregorian calendar
+    long d = long ((c - 122.1) / 365.25);
+    long e = 365 * d + (d/4);
+    long f = long ((c - e) / 30.6001);
+    int day = c - e - long(30.6001 * f);
+    int month = f - 1 - 12 * (f / 14);
+    int year = d - 4715 - ((7 + month) / 10);
+    double hour = 24.0 * (mjd - floor(mjd));
+
+    QString date = QString::number(year)+"-"+QString::number(month)+"-"+QString::number(day)+"T";
+    QString time = decimalToDms(hour);
+
+    return date+time;
+}
+
 double Splitter::dateobsToMJD()
 {
     if (!checkFormatDATEOBS()) {
@@ -1029,7 +1508,7 @@ double Splitter::dateobsToLST()
     double dfrac = ut / 24.;
     double d = dwhole + dfrac;
 
-    double lst = 100.46 + 0.985647*d + instData.obslong + 15.*ut;
+    double lst = 100.46 + 0.985647*d + instData.obslon + 15.*ut;
 
     // LST must be between 0 and 360 degrees
     int idummy = (int)(lst/360.);
