@@ -72,6 +72,7 @@ void Splitter::correctOverscan()
     }
 
     // Update the saturation value
+    //    qDebug() << saturationValue << overscanEffective;
     saturationValue -= overscanEffective;
 }
 
@@ -279,14 +280,21 @@ void Splitter::applyMask(int chip)
 {
     if (!successProcessing) return;
 
-    if (!checkCorrectMaskSize(chip)) return;
-
     if (!MEFpastingFinished) return;
     else {
         // Must reset 'chip' such that it counts true CCDs, not the extensions with the individual amplifiers (e.g., for GMOS and SOI)
         chip = chip / numAmpPerChip;
     }
 
+    // Override: PISCO chip can be different than zero (other than grond) because we had to maintain its chip id for the chip-dependent pasting
+    if (instData.name == "PISCO@LCO") chip = 0;       // (single detector, effectively)
+    if (!checkCorrectMaskSize(chip)) {
+        //        QDir unknownFile(path+"/InconsistentGeometry");
+        //        unknownFile.mkpath(path+"/InconsistentGeometry/");
+        //        moveFile(fileName, path, path+"/InconsistentGeometry/");
+        return;
+    }
+
     if (instData.name == "LIRIS_POL@WHT") return;           // Further cut-outs happen in writeImage(); mask geometry in camera.ini applies only once these have happened.
     //    if (instData.name == "GROND_NIR@MPGESO") return;  // Further cut-outs happen in writeImage(); mask geometry in camera.ini applies only once these have happened.
     if (instNameFromData == "GROND_NIR@MPGESO") return;     // Further cut-outs happen in writeImage(); mask geometry in camera.ini applies only once these have happened.
@@ -363,6 +371,14 @@ void Splitter::correctNonlinearity(int chip)
 
     // Extract coefficients for the current chip
     QVector<float> coeffs = nonlinearityCoefficients[chip];
+    // Apply nonlinearity correction
+    for (auto &pixel : dataCurrent) {
+        pixel = polynomialSum(pixel, coeffs);
+    }
+    // Update saturation value
+    saturationValue = polynomialSum(saturationValue, coeffs);
+
+    // Individual implementations
 
     // Average nonlinearity correction
     // http://instrumentation.obs.carnegiescience.edu/FourStar/Documents/FourStar_Documentation.pdf
@@ -385,14 +401,65 @@ void Splitter::correctNonlinearity(int chip)
         for (auto &pixel : dataCurrent) {
             pixel = pixel * (1.+A*pow(pixel,1.5));
         }
+        saturationValue *= (1.+A*pow(saturationValue,1.5));
     }
 
-    for (auto &pixel : dataCurrent) {
-        pixel = polynomialSum(pixel, coeffs);
+    // http://csp2.lco.cl/manuals/dup_dc_linearity.php
+    if (instData.name == "Direct_2k_DUPONT@LCO") {
+        QString speed = "";
+        searchKeyValue(headerDictionary.value("SPEED"), speed);
+
+        float c1 = 1.0;
+        float c2 = 0.0;
+        float c3 = 0.0;
+        float c4 = 0.0;
+        if (speed == "Turbo") {
+            c2 = 0.09907;
+            c3 = -0.06151;
+            c4 = 0.01522;
+        }
+        if (speed == "Fast") {
+            c2 = 0.05022;
+            c3 = -0.02814;
+            c4 = 0.00733;
+        }
+        for (auto &pixel : dataCurrent) {
+            float x = c1 + c2*(pixel/32000.0) + c3*pow(pixel/32000.0,2) + c4*pow(pixel/32000.0,3);
+            pixel /= x;
+        }
+        float x = c1 + c2*(saturationValue/32000.0) + c3*pow(saturationValue/32000.0,2) + c4*pow(saturationValue/32000.0,3);
+        saturationValue /= x;
     }
 
-    // Update saturation value
-    saturationValue = polynomialSum(saturationValue, coeffs);
+    // http://csp2.lco.cl/manuals/swo_nc_linearity.php (using mean values)
+    // NOTE: linearity test page does not specify for which readout mode it applies
+    if (instData.name == "Direct_4k_SWOPE@LCO") {
+        float c1 = 1.0;
+        float c2 = 0.0;
+        float c3 = 0.0;
+        if (chip == 0) {
+            c2 = -0.03443;
+            c3 = 0.006657;
+        }
+        if (chip == 1) {
+            c2 = -0.01557;
+            c3 = 0.003386;
+        }
+        if (chip == 2) {
+            c2 = -0.01329;
+            c3 = 0.003314;
+        }
+        if (chip == 3) {
+            c2 = -0.01971;
+            c3 = 0.004700;
+        }
+        for (auto &pixel : dataCurrent) {
+            float x = c1 + c2*(pixel/32000.0) + c3*pow(pixel/32000.0,2);
+            pixel /= x;
+        }
+        float x = c1 + c2*(saturationValue/32000.0) + c3*pow(saturationValue/32000.0,2);
+        saturationValue /= x;
+    }
 }
 
 void Splitter::convertToElectrons(int chip)
@@ -401,9 +468,8 @@ void Splitter::convertToElectrons(int chip)
 
     if (!MEFpastingFinished) return;
 
-
-    // Update saturation value
-    saturationValue *= gain[chip];
+    // Update saturation value          // done in buildTheliHeaderGAIN(int chip)
+    // saturationValue *= gain[chip];
 
     // cameras with multiple readout channels AND different gain per channel (but no overscan pasted into the data section)
     // WARNING: ASSUMPTIONS: applies to the size defined in camera.ini
@@ -419,22 +485,28 @@ void Splitter::convertToElectrons(int chip)
                     if (i>=1023 && j>=1024) pixel *= channelGains[3];   // upper right quadrant
                 }
             }
+            gainForSaturation = minVec_T(channelGains);    // SATURATE keyword shall reflect lowest number at which saturation occurs
         }
     }
 
     //    if (instData.name == "GROND_NIR@MPGESO") {
     if (instNameFromData == "GROND_NIR@MPGESO") {
         gain[chip] = 1.0;   // Gain is fixed in writeImageIndividual();
+        saturationValue *= gainForSaturation;
         return;
     }
     if (instData.name == "SuprimeCam_200808-201705@SUBARU") {
+        saturationValue *= gainForSaturation;
         return;
     }
     if (multiChannelMultiExt.contains(instData.name)) {
-        // GMOS, SOI
+        // GMOS, SOI, MOSAIC-II_16@CTIO, etc
+        saturationValue *= gainForSaturation;
         return;
     }
 
+    saturationValue *= gainForSaturation;
+
     for (auto &pixel : dataCurrent) {
         pixel *= gain[chip];
     }
