@@ -73,7 +73,7 @@ bool Splitter::searchKeyValue(const QStringList &possibleKeyNames, int &value)
 {
     bool inExtHeader = false;
     bool inPrimaryHeader = searchKeyInHeaderValue(possibleKeyNames, primaryHeader, value);
-    if (possibleKeyNames.contains("NAMPS")) qDebug() << fileName << possibleKeyNames << primaryHeader.length() << inPrimaryHeader << value;
+    if (possibleKeyNames.contains("NAMPS")) qDebug() << __func__ << fileName << possibleKeyNames << primaryHeader.length() << inPrimaryHeader << value;
     if (!inPrimaryHeader) inExtHeader = searchKeyInHeaderValue(possibleKeyNames, extHeader, value);
 
     if (!inExtHeader && !inPrimaryHeader) return false;
@@ -190,9 +190,18 @@ bool Splitter::searchKeyInHeaderCRVAL(const QString searchKey, const QStringList
                         || instData.name == "SITe@TLS") {
                     crval = QString::number(15.*crval.toDouble(), 'f', 12);
                 }
+                // And sometimes it may under- or overrun the [0,360] degree boundary
+                if (searchKey == "CRVAL1") {
+                    if (crval.toDouble() > 360.) crval = QString::number(crval.toDouble()-360., 'f', 12);
+                    else if (crval.toDouble() < 0.) crval = QString::number(crval.toDouble()+360., 'f', 12);
+                }
+                if (searchKey == "CRVAL2") {
+                    if (crval.toDouble() > 90.) crval = "90.0";
+                    else if (crval.toDouble() < -90.) crval = "-90.0";
+                }
 
                 // The William Herschel Telescope has its own funny "apertures" ...
-                if (instData.name.contains("@WHT")) {
+                if (instData.name.contains("@WHT") && !instData.name.contains("PF_QHY")) {
                     double ra_off1 = 0.;
                     double ra_off2 = 0.;
                     double dec_off1 = 0.;
@@ -209,6 +218,23 @@ bool Splitter::searchKeyInHeaderCRVAL(const QString searchKey, const QStringList
                     if (searchKey == "CRVAL2") crval = QString::number(crval.toDouble() + dec_off1 + dec_off2, 'f', 12);
                 }
 
+                // GSAOI must apply offsets, too
+                if (instData.name.contains("GSAOI")) {
+                    double ra_off = 0.;
+                    double dec_off = 0.;
+                    QStringList ra = {"RAOFFSET"};
+                    QStringList dec = {"DECOFFSE"};
+                    searchKeyInHeaderValue(ra, inputHeader, ra_off);
+                    searchKeyInHeaderValue(dec, inputHeader, dec_off);
+                    if (searchKey == "CRVAL1") {
+                        double dec_pointing = 0.;
+                        QStringList decpoint = {"DEC"};
+                        searchKeyInHeaderValue(decpoint, inputHeader, dec_pointing);
+                        crval = QString::number(crval.toDouble() + ra_off / 3600. / cos(dec_pointing*rad), 'f', 12);
+                    }
+                    if (searchKey == "CRVAL2") crval = QString::number(crval.toDouble() + dec_off/3600., 'f', 12);
+                }
+
                 // Construct the key card
                 QString newCard = searchKey;
                 newCard.resize(8, ' ');          // Keyword must be 8 chars long
